/**\
 Rexx program to run BGFAX and then call Maximus

   Written by Brenda Moon.

   BGFAX is a shareware DOS program written by B.J. Guillot which determines 
   if incoming calls are FAX or DATA.

   If a FAX call is detected BGFAX receives the FAX.
   If a DATA call is detected BGFAX creates a DOBBS.BAT with one line as follows:

   D:\BGFAX\EXEBBS.BAT 14400 2 0 /ARQ

   These are:
   PathToEXEBBS.BAT     DCE Port TimeToNextEvent ExtendedConnectDetails

   Maximus/2 requires the file handle of an open com port, not the port number as a
   command line parameter.  In order to provide this, the com port is opened by this
   Rexx program as part of it's initialisation and the returned com handle used when
   calling Maximus/2.  This is only possible using the SIO drivers with the "-"
   parameter to allow sharing of the com port.  When sharing the com port it is
   essential to make sure that only one task is writing to it at a time, otherwise the
   remote user will see a very mixed up screen.

   Using RxComOpen & RxComClose from Ian Timms Rexx Comms library.
   This requires that RXASYNC.DLL be in the DLL path.  

   BGFAX in frontend mode uses it's BGFAX.CNF file to determine which line it is on, 
   so this must match the ComPort constant set in here.

   In rear end mode BGFAX has the ComPort as a parameter so it is possible to run one
   line in FrontEnd mode and another in rearend mode with the same BGFAX.CNF

  *** NOT TESTED WITH NEW RXASYNC.DLL or recent BGFAX versions  **********
   
\**/
parse source . . Spec

if rxfuncquery(sysloadfuncs) \= 0 then
  do
    call rxfuncadd 'sysloadfuncs','rexxutil','sysloadfuncs'
    call sysloadfuncs
  end

/*      Constants,      */
False    = 0
True     = 1
Exist           = 1
NotExist        = 0
StartTime      = time()
StartDate      = date()
Message         = '(none)'
Tab             = d2c(9)
Esc = d2c(27)||'['

ProgramName    = Esc||'1;33m'||filespec('NAME', Spec)||Esc||'0m'

CurrentDir     = directory()


Version         = 'Version 1.02'
DateModified    = '(9/9/94)'

/* Task parameters */
ComPort = 'COM2'
TaskNumber = 2
LockRate = 19200

/* Load Comms Functions */

Call RxFuncAdd 'RxComLoadFuncs', 'RXASYNC', 'RxComLoadFuncs'
Call RxComLoadFuncs
Say 'RxComLoadFuncs is complete.'

/*  Display program title and version  */

call syscls
call DisplayTitle

/*      Perform test for error log file path test,       */
LogDir         = 'D:\BINKLEY\LOGS'

if directory(LogDir) <> LogDir then do
   say directory(LogDir)||Tab||LogDir
   say
   say Tab||' *******CANNOT CREATE LOG FILE*******'
   say Tab||' *******ABORTING '||ProgramName||'*******'
   exit(1)
end

/**\
 PATHS,
      NOTE TO USER: The function CheckPath('[Drive]:\[Path]') tests
                    for the existence of '[Drive]:\[Path]' and sets
                    the appropriate variable to it ONLY IF IT DOES EXIST.
                    Therefore, YOU must edit the '[Drive]:\[Path]' arguments
                    of each call to CheckPath to reflect YOUR hard disk.

               --MAKE A COPY OF THIS FILE BEFORE YOU EDIT IT--
\**/
MaxDir    = CHECKPATH('D:\MAX')
BGFAXDir  = CHECKPATH('C:\BGFAX')

/* Open Com port */

PortHandle = ''

rc = RxComOpen( ComPort, 'PortHandle' )
if rc <> 0
   then Call ErrorProc 'RxComOpen failed with rc=>'rc'<'
   else Say 'RxComOpen opened the device handle=>'PortHandle'<'

/* Setup BBSCMD Queue for communicating with MailProc.CMD   */

FlagQueue = CheckQueue('BBSCMD')

/* Start BGFAX and loop until user selects exit */

Finished = False
Do Until Finished

  rc = Directory(BGFAXDir)
/* 
 For some reason, although Maximus hangs up the phone, the modem
 goes back offhook when it gets back to a OS/2 or DOS command
 This means that when it gets back to BGFAX it is offhook, and
 unless you have ATH0 in your BGFAX init string it will think
 the modem is ringing.
*/

  'copy dobbs.bat dobbs.old'     /* when running with FAX=DEBUG     */
  'del dobbs.bat'                /* BGFAX doesn't seem to write     */
                                 /* dobbs.bat if one already exists */


  'bgfax'                        /* OS/2 automatically changes  */
                                 /* session to DOS              */

  select
  when rc = 3 | rc = 2 then      /* Incoming BBS call           */
       Call DataCall 

  when rc = 4 then               /* Have routine call faxrcvd   */
       Call FaxRcvd              /* to print faxes if wanted    */
                                 
  when rc = 0 then               /* serious error or user       */
       Finished = True           /* pressed <Esc> to exit BGFAX */
  
  when rc = 1 then               /* recycle - do nothing        */
       NOP

  when rc = 5 then               /* successful fax send....     */
       NOP

  otherwise                      /* unexpected errorlevel       */

       Call ErrorProc 'Unexpected errorlevel from BGFAX: '||rc
 
  end /* select */

end /* do */

rc = RxComClose( PortHandle )
if rc <> 0
   then say 'RxComClose failed with rc=>'rc'<'
   else say 'RxComClose closed the device (PortHandle = '||PortHandle||' )'

call RxComDropFuncs
Say 'RxComDropFuncs is complete.'

Call ExitProgram

Exit 0

End


/**\
    Sub-routine DataCall
    This routine reads the dobbs.bat file created by BGFAX to 
    determine the DCE speed and then calls Maximus.
\**/
DataCall:

FirstLine = LineIn('dobbs.bat',1,1)
rc=LineOut('dobbs.bat')

Parse Var FirstLine . DCERate . TimeLeft ExtInfo

If TimeLeft = 0 then TimeLeft = 999

LogFile = LogDir||'\Bunyip'||TaskNumber||'.log'

message = '* '||left(date(),Length(date())-5) time()||' BBSFAX s='||LockRate
message = message||' b='||DCERate||' t='||TimeLeft||' e='||ExtInfo
message = message||' p='||PortHandle
call lineout LogFile, Message
call lineout LogFile

rc = Directory(MaxDir)

MaxParams = '-b'||DCERate||' -p'||PortHandle||' -s'||LockRate||' -n0'
MaxParams = MaxParams||TaskNumber||' -l'||LogFile||' -t'||TimeLeft
MaxParams = MaxParams||' -z\pipe\bbs'||TaskNumber

'maxp.exe '||MaxParams

if rc = 12 | rc = 11 then do   /* Mail to process  */
    call SendQMsg 'SQUISH'
end /* if */

call SendQMsg 'LCALL'

Return

/**\
    Sub-routine FaxRcvd
    This routine prints received faxes  (dummy routine at this stage)
    BGFAX appends to DOPRINT.BAT whenever a fax is received. 
    The format of DOPRINT.BAT is:
       CALL C:\BGFAX\EXEPRINT.BAT C:\BGFAX\FAX0085.ZFX
       CALL C:\BGFAX\EXEPRINT.BAT C:\BGFAX\FAX0086.ZFX
    DOPRINT.BAT should be deleted when the faxes are printed.
\**/
FaxRcvd:
Return

/**\
       Sub-routine: ErrorProc
       This routine writes the passed argument to [name of program].LOG
\**/
ErrorProc: Procedure expose Spec LogDir Tab ProgramName CurrentDir
parse arg ErrorMsg

pName   = filespec('NAME', Spec)

tName   = left(pName, pos('.', pName) - 1)
ErrorLogFile = LogDir||'\'||tName||'.LOG'

rc      = beep(880, 1000)

say; say ' *******'||errormsg||'*******'
say; say ' LOGGING ERROR TO '||ErrorLogFile

call lineout ErrorLogFile, ErrorMsg||Tab||date()||Tab||time()
call lineout ErrorLogFile

say ' Terminating '||ProgramName

rc      = directory(CurrentDir)
exit(1)

/**\
       Sub-routine: DisplayTitle
       Displays program title line
\**/
DisplayTitle:
say
VertLine = Esc||'0;30;46m'||Esc||'1;37;46m'
call charout , ' '||Esc||'1;46m '||ProgramName||Esc||'1;46m '||VertLine
parse value SysCurPos() with . cols
call charout , center(Version||'  '||DateModified, 79 - 23 - cols)
call charout , VertLine||' Brenda Moon & M.N.S. '||Esc||'0m'
say

return
exit 1

/**\
       Sub-routine CheckPath
       This routine accepts a path argument and tests
       for its existance.
\**/
CheckPath: procedure
parse arg PathName

if directory(PathName) = '' then
   call ErrorProc 'Path error: '||pathname||' does not exist!'

return PathName
exit 1

/**\
       Sub-routine CheckQueue
       This routine accepts the name of the message queue used to
       pass flags to this program. The queue is created if needed and
       set as the current session message queue.
\**/
CheckQueue: procedure
parse arg QueueName

QueueTest      = RxQueue('CREATE', QueueName)

if QueueTest <> QueueName then
   rc   = RxQueue('DELETE', QueueTest)

OldQueue       = RxQueue('SET', QueueName)

return QueueName
exit 1

/**\
       SendQMsg
       Pushes a message into the queue
\**/
SendQMsg: procedure
parse arg Message

call RxQueue 'SET', 'BBS_CMD'
queue Message

call SysSleep 1

return

exit 1

/**\
     ExitProgram
     Performs all cleanup and displays session record.
\**/
ExitProgram:
Procedure expose CurrentDir ProgramName Version DateModified StartDate StartTime Esc Tab

rc = directory(CurrentDir)

beep(1660, 70)

call SysCls
call SysCurState 'OFF'
call DisplayTitle
say
say Tab||'Activated:'||Tab||StartDate||d2c(9)||StartTime
say Tab||'Terminated:'||Tab||date()||Tab||time()
say

say ' '||ProgramName||Esc||'1;31m terminated.'||Esc||'0m'

return

exit 1

