/*****************************************************************************/
/*                                                                           */
/*  MODULE         RXASYNC.c                                                 */
/*                                                                           */
/*  VERSION        Version 1.5 - 23rd May 1994                               */
/*                                                                           */
/*  COPYRIGHT      Copyright (c) 1993, 1994 by Crucial Applications          */
/*                              All rights reserved.                         */
/*                                                                           */
/*  AUTHOR         Ian Timms                                                 */
/*                                                                           */
/*  DESCRIPTION                                                              */
/*                                                                           */
/*    The enclosed REXX DLL comprises a set of functions which are used to   */
/*    to extend the REXX language by providing FULL exposure of the ASYNC    */
/*    communications API. In addition a number of higher level functions     */
/*    are also provided to simplify the task of enabling REXX to be used     */
/*    as a scripting language for serial communications control.             */
/*                                                                           */
/*    They are by no means perfect but do function sufficiently well for the */
/*    author who uses them to provide a means of automating the collection   */
/*    of mail from the BBS.  If you have any comments or suggestions for     */
/*    improvement then I'd be glad to hear from you.                         */
/*                                                                           */
/*    The author can be contacted (in order of preference) at:               */
/*                                                                           */
/*         Bunyips Cave BBS    3:633/379                                     */
/*                                                                           */
/*         Internet            itimms@ariel.ucs.unimelb.edu.au               */
/*                                                                           */
/*         Compuserve          100236,1404                                   */
/*                                                                           */
/*         Snail mail          Ian Timms,                                    */
/*                             13 Cowper Street                              */
/*                             Footscray, Victoria,                          */
/*                             Australia, 3011                               */
/*                                                                           */
/*  FUNCTIONS                                                                */
/*                                                                           */
/*       House keeping                                                       */
/*                                                                           */
/*    RxAsyncLoadFuncs     -   Load the functions into REXX                  */
/*    RxAsyncDropFuncs     -   Drop the functions from REXX                  */
/*                                                                           */
/*       Process control                                                     */
/*                                                                           */
/*    RxAsyncPriority      -   Set Rexx process priority                     */
/*    RxAsyncSleep         -   Wait for some defined period                  */
/*                                                                           */
/*       ASYNC IOCTL interface                                               */
/*                                                                           */
/*    RxAsyncGetDcbInfo    -   Get the device control block info             */
/*    RxAsyncSetDcbInfo    -   Set the device control block info             */
/*    RxAsyncGetEnhParm    -   Get the enhanced parameters                   */
/*    RxAsyncSetEnhParm    -   Set the enhanced parameters                   */
/*    RxAsyncGetLnCtrl     -   Get the comms port settings                   */
/*    RxAsyncSetLnCtrl     -   Set the comms port settings                   */
/*    RxAsyncSetBreakOff   -   Turn break off                                */
/*    RxAsyncSetBreakOn    -   Turn break on                                 */
/*    RxAsyncTxImmediate   -   Transmit byte immediately                     */
/*    RxAsyncTxStop        -   Stop transmission (ala XOFF)                  */
/*    RxAsyncTxStart       -   Start transmission (ala XON)                  */
/*    RxAsyncGetComStatus  -   Get COM status                                */
/*    RxAsyncGetTxStatus   -   Get Transmit status                           */
/*    RxAsyncGetMdmOutput  -   Get modem output signals                      */
/*    RxAsyncSetMdmOutput  -   Set modem output signals                      */
/*    RxAsyncGetMdmInput   -   Get modem input signals                       */
/*    RxAsyncGetRxCount    -   Get size and number of chars in RX queue      */
/*    RxAsyncGetTxCount    -   Get size and number of chars in TX queue      */
/*    RxAsyncGetComError   -   Get the COM Error Word                        */
/*    RxAsyncGetComEvent   -   Get the COM Event Word                        */
/*                                                                           */
/*       Extensions                                                          */
/*                                                                           */
/*    RxAsyncOpen          -   Open the specified comms port                 */
/*    RxAsyncClose         -   Close the specified comms port                */
/*    RxAsyncWait          -   Wait for receipt of input                     */
/*    RxAsyncRead          -   Read data and wait if necessary               */
/*    RxAsyncWrite         -   Write data to the comms port                  */
/*    RxAsyncPrint         -   Print string on stdout                        */
/*    RxAsyncCarrier       -   Test for Data Carrier Detect (DCD)            */
/*    RxAsyncBreak         -   Send break for defined period                 */
/*    RxAsyncDcbInfo       -   Print the device control block info           */
/*                                                                           */
/*       File handling                                                       */
/*                                                                           */
/*    RxAsyncFileExists    -   Test for file(s) existence                    */
/*    RxAsyncFileDelete    -   Delete specified file                         */
/*    RxAsyncFileCopy      -   Copy specified file                           */
/*    RxAsyncFileMove      -   Move specified file                           */
/*    RxAsyncFileRename    -   Rename specified file                         */
/*    RxAsyncFileNameIs    -   Determine file name                           */
/*    RxAsyncFilePathIs    -   Determine file path                           */
/*    RxAsyncDirectory     -   Query/Change directory/drive                  */
/*                                                                           */
/*  FILES                                                                    */
/*                                                                           */
/*    READ.ME      -  Documentation (such as it is)                          */
/*    RXASYNC.C    -  Source code for the DLL                                */
/*    RXASYNC.H    -  Header file                                            */
/*    RXASYNC.DEF  -  Definitions file for linker                            */
/*    RXASYNC.DEP  -  Dependancy file make                                   */
/*    RXASYNC.MAK  -  Make file                                              */
/*    RXASYNC.DLL  -  The final product                                      */
/*    RXASYNC.CMD  -  Rexx script showing basic usage of DLL                 */
/*                                                                           */
/*  LICENSE                                                                  */
/*                                                                           */
/*    This Program is released to the public, you may use this Program in    */
/*    any way you see fit, pursuant to the following restrictions:           */
/*                                                                           */
/*    o  You may not sell the Program or use this code _in_ a commercial     */
/*       application nor may you distribute this Program as a part of any    */
/*       such application without the express written permission of the      */
/*       author;                                                             */
/*                                                                           */
/*    o  The Program may be freely used for non commercial and educational   */
/*       purposes, privately and by registered charities and non-profit      */
/*       organisations;                                                      */
/*                                                                           */
/*    o  You may not charge a fee for distribution of the Program;           */
/*                                                                           */
/*    o  The Program shall be supplied in its original, unmodified form, as  */
/*       distributed by the author, which includes this source code.         */
/*                                                                           */
/*  DISCLAIMER                                                               */
/*                                                                           */
/*    Use of this Program and/or documentation for any purpose whatsoever    */
/*    constitutes your unqualified acceptance of the following statements.   */
/*                                                                           */
/*    The author, Ian Timms, makes no warranty or representation that the    */
/*    Program or Documentation will be error free.  The author disclaims     */
/*    any warranties, either express or implied, including but not limited   */
/*    to any implied warranty of merchantability or fitness for any          */
/*    particular purpose.                                                    */
/*                                                                           */
/*    The author has no obligation to provide service, defect correction, or */
/*    any maintenance for the Program.  The author has no obligation to      */
/*    supply any Program updates or enhancements to you even if such are or  */
/*    later become available.                                                */
/*                                                                           */
/*    In no event will the author be liable for any damages whatsoever       */
/*    (including without limitation damages for loss of business profits,    */
/*    business interruption, loss of business information or other           */
/*    incidental or consequential damages) arising out of the use of,        */
/*    interruption in the use of, or inability to use the Program, even if   */
/*    the author has been advised of any possibility or likelihood of such   */
/*    damages.                                                               */
/*                                                                           */
/*    The user agrees to take full responsibility for the selection of and   */
/*    any use whatsoever made of the Program and Documentation such as it    */
/*    exists.                                                                */
/*                                                                           */
/*  NOTES                                                                    */
/*                                                                           */
/*    Online timeout etc. is checked in hundredths of a second as this is    */
/*    the maximum granularity available with standard OS/2 calls.  Use of a  */
/*    high resolution timer might enhance this aspect significantly but I    */
/*    have not yet found a need for this. The MyDiffTime routine which is    */
/*    used for handling timeout calculation can only calulate a maximum      */
/*    period difference to within 1 week which should be sufficient.         */
/*                                                                           */
/*    The routines themselves do NOT check for the loss of carrier.  Such    */
/*    checking as exists in the communications drivers is relied upon for    */
/*    this. I use the NO CARRIER response from the modem and this does the   */
/*    trick reasonably well. There is now an RxAsyncCarrier function which   */
/*    will tell you the carrier status, ideal for loop control.              */
/*                                                                           */
/*    Nulls are NOT stripped from any input read from the device. I had      */
/*    initially intended doing this but have so far not implemented this     */
/*    facility, even though the majority of the code is already there. I     */
/*    should also note that it would only strip nulls from the scripted      */
/*    side of things, that is to say I would NOT tell the device driver to   */
/*    do it, that would be an unwise move for reasons which should become    */
/*    obvious when you think about the effect it would have on an external   */
/*    protocol.                                                              */
/*                                                                           */
/*    Please use the RxAsyncPrint function to display all "stdout" style     */
/*    output, in this way you will correctly see the error results returned  */
/*    by the routines should a failure occur. It also avoids the problem of  */
/*    REXX adding <cr><lf> to everything.                                    */
/*                                                                           */
/*    In order to make sure that output containing ANSI escape sequences is  */
/*    correctly handled you will need to load the ANSI.sys driver in your    */
/*    CONFIG.sys file.                                                       */
/*                                                                           */
/*    The file handling functions which I have added are NOT replacements    */
/*    for the standard OS/2 command line functions. They do NOT handle       */
/*    wildcard nameing (except for FileExists) and unpredictable results     */
/*    may occur if you attempt to do so.  What the functions WILL do is      */
/*    enable you to easily move/copy/delete/rename and find a file without   */
/*    having to shell out to the CMD.EXE command processor.  The FileMove    */
/*    function has also been written so that it is possible to move a file   */
/*    across drives, something the standard MOVE command will not allow.     */
/*                                                                           */
/*    I have now implemented propper device control block handling which     */
/*    permits setting of more reasonable timeouts using the standard OS/2    */
/*    COM.SYS driver. The default values of 1 minute can now be overridden   */
/*    after the communications port is opened by using the RxAsyncSetDcbInfo */
/*    function.                                                              */
/*                                                                           */
/*    If you use the COM.SYS driver supplied with OS/2 2.x GA and elect NOT  */
/*    to set read and write timeouts using the RxAsyncSetDcbInfo function    */
/*    then you'll find the functionality of these routines a little lacking. */
/*                                                                           */
/*    To control handshaking, extended buffering etc. etc. I recommend that  */
/*    you use the MODE command to set up your port appropriately before      */
/*    using these routines. You can also do this using the RxAsyncSetDcbInfo */
/*    and RxAsyncSetLnCtrl functions but generally it is much simpler to use */
/*    the MODE command. Locking your pc<->modem port speed is highly         */
/*    recommended.                                                           */
/*                                                                           */
/*    The following MODE setting is recommended to be placed in your         */
/*    STARTUP.CMD file for best results:                                     */
/*                                                                           */
/*       MODE COM4:19200,N,8,1,TO=OFF,XON=OFF,        full duplex            */
/*                             OCTS=ON,RTS=HS,        hardware CTS/RTS       */
/*                             DTR=ON,                DTR control mode       */
/*                             IDSR=OFF,ODSR=OFF,     disable DSR checking   */
/*                             BUFFER=ON              bufferred              */
/*                                                                           */
/*    Note that I have DSR checking disabled, my modem does not appear to    */
/*    handle the DSR signal correctly, if you are not getting any response   */
/*    from yours then the same may well be true. Also having this enabled    */
/*    may cause unpredictable responses when using the SIO drivers.          */
/*                                                                           */
/*    You should also use the following Hayes or equivalent modem settings:  */
/*                                                                           */
/*        E0, Q0, V1, X4 or X6, &C1, &D2                                     */
/*                                                                           */
/*  HISTORY                                                                  */
/*                                                                           */
/*    14-Aug-1993  Original release.                                         */
/*    24-Aug-1993  Added carrier test function. Thanks to M.N.Stephenson.    */
/*    04-Sep-1993  Added file handling functions.                            */
/*    02-Nov-1993  Added RxAsyncGetDcbInfo function.                         */
/*    06-Nov-1993  Added RxAsyncBreak function.                              */
/*    18-Dec-1993  Changed RxAsyncGetDcbInfo to RxAsyncDcbInfo               */
/*    18-Dec-1993  Improved documentation                                    */
/*    11-Mar-1994  Permit passing empty string to RxAsyncWrite.              */
/*    11-Mar-1994  Changed both read and write timeouts to 1 second.         */
/*    11-Mar-1994  Added RxAsyncGetDcbInfo function.                         */
/*    11-Mar-1994  Added RxAsyncGetEnhParm function.                         */
/*    03-May-1994  Added generic Str2Num function.                           */
/*    03-May-1994  Added RxAsyncSetDcbInfo function.                         */
/*    03-May-1994  Added RxAsyncSetEnhParm function.                         */
/*    03-May-1994  Removed forcing of full duplex and read/write timeouts.   */
/*    03-May-1994  Changed name to RXASYNC.                                  */
/*    03-May-1994  Cleaned up parameter validations.                         */
/*    20-May-1994  Added extra modem control and signal query functions.     */
/*    22-May-1994  Prepared for distribution.                                */
/*                                                                           */
/*  FUTURES                                                                  */
/*                                                                           */
/*    Internalise the whole shebang into an EXE scripting tool. Eventually!  */
/*                                                                           */
/*****************************************************************************/

/*********************************************************************/
/* Includes.                                                         */
/*********************************************************************/

#define  INCL_DOS
#define  INCL_DOSDEVIOCTL
#define  INCL_ERRORS
#define  INCL_REXXSAA
#define  _DLL
#define  _MT

#include <os2.h>
#include <rexxsaa.h>
#include <string.h>
#include <time.h>
#include <memory.h>
#include <stdlib.h>
#include <stdio.h>
#include "rxasync.h"

/*********************************************************************/
/* Declare exported functions as REXX functions.                     */
/*********************************************************************/

RexxFunctionHandler RxAsyncLoadFuncs;
RexxFunctionHandler RxAsyncDropFuncs;

RexxFunctionHandler RxAsyncPriority;
RexxFunctionHandler RxAsyncSleep;

RexxFunctionHandler RxAsyncGetDcbInfo;
RexxFunctionHandler RxAsyncSetDcbInfo;
RexxFunctionHandler RxAsyncGetEnhParm;
RexxFunctionHandler RxAsyncSetEnhParm;
RexxFunctionHandler RxAsyncGetLnCtrl;
RexxFunctionHandler RxAsyncSetLnCtrl;
RexxFunctionHandler RxAsyncSetBreakOff;
RexxFunctionHandler RxAsyncSetBreakOn;
RexxFunctionHandler RxAsyncTxImmediate;
RexxFunctionHandler RxAsyncTxStop;
RexxFunctionHandler RxAsyncTxStart;
RexxFunctionHandler RxAsyncGetComStatus;
RexxFunctionHandler RxAsyncGetTxStatus;
RexxFunctionHandler RxAsyncGetMdmOutput;
RexxFunctionHandler RxAsyncSetMdmOutput;
RexxFunctionHandler RxAsyncGetMdmInput;
RexxFunctionHandler RxAsyncGetRxCount;
RexxFunctionHandler RxAsyncGetTxCount;
RexxFunctionHandler RxAsyncGetComError;
RexxFunctionHandler RxAsyncGetComEvent;

RexxFunctionHandler RxAsyncOpen;
RexxFunctionHandler RxAsyncClose;
RexxFunctionHandler RxAsyncWait;
RexxFunctionHandler RxAsyncRead;
RexxFunctionHandler RxAsyncWrite;
RexxFunctionHandler RxAsyncPrint;
RexxFunctionHandler RxAsyncCarrier;
RexxFunctionHandler RxAsyncBreak;
RexxFunctionHandler RxAsyncDcbInfo;

RexxFunctionHandler RxAsyncFileExists;
RexxFunctionHandler RxAsyncFileDelete;
RexxFunctionHandler RxAsyncFileCopy;
RexxFunctionHandler RxAsyncFileMove;
RexxFunctionHandler RxAsyncFileRename;
RexxFunctionHandler RxAsyncFileNameIs;
RexxFunctionHandler RxAsyncFilePathIs;
RexxFunctionHandler RxAsyncDirectory;

/*********************************************************************/
/* RxFncTable                                                        */
/*   Array of names of the RXASYNC functions.                        */
/*   This list is used for registration and deregistration.          */
/*********************************************************************/

static PSZ  RxFncTable[] =
   {
      "RxAsyncLoadFuncs"
   ,  "RxAsyncDropFuncs"
   ,  "RxAsyncPriority"
   ,  "RxAsyncSleep"
   ,  "RxAsyncGetDcbInfo"
   ,  "RxAsyncSetDcbInfo"
   ,  "RxAsyncGetEnhParm"
   ,  "RxAsyncSetEnhParm"
   ,  "RxAsyncGetLnCtrl"
   ,  "RxAsyncSetLnCtrl"
   ,  "RxAsyncSetBreakOff"
   ,  "RxAsyncSetBreakOn"
   ,  "RxAsyncTxImmediate"
   ,  "RxAsyncTxStop"
   ,  "RxAsyncTxStart"
   ,  "RxAsyncGetComStatus"
   ,  "RxAsyncGetTxStatus"
   ,  "RxAsyncGetMdmOutput"
   ,  "RxAsyncSetMdmOutput"
   ,  "RxAsyncGetMdmInput"
   ,  "RxAsyncGetRxCount"
   ,  "RxAsyncGetTxCount"
   ,  "RxAsyncGetComError"
   ,  "RxAsyncGetComEvent"
   ,  "RxAsyncOpen"
   ,  "RxAsyncClose"
   ,  "RxAsyncWait"
   ,  "RxAsyncRead"
   ,  "RxAsyncWrite"
   ,  "RxAsyncPrint"
   ,  "RxAsyncCarrier"
   ,  "RxAsyncBreak"
   ,  "RxAsyncDcbInfo"
   ,  "RxAsyncFileExists"
   ,  "RxAsyncFileDelete"
   ,  "RxAsyncFileCopy"
   ,  "RxAsyncFileMove"
   ,  "RxAsyncFileRename"
   ,  "RxAsyncFileNameIs"
   ,  "RxAsyncFilePathIs"
   ,  "RxAsyncDirectory"
   };

/*********************************************************************/
/* SUPPORT FUNCTIONS                                                 */
/*                                                                   */
/*    DosDevIOCtl wraparound function and related                    */
/*     - CallDevice                                                  */
/*     - ModemInput                                                  */
/*     - ModemOutput                                                 */
/*     - PrtDcbInfo                                                  */
/*                                                                   */
/*    File handling routines                                         */
/*     - FileExists                                                  */
/*     - FileDelete                                                  */
/*     - FileCopy                                                    */
/*     - FileRename                                                  */
/*     - FileMove                                                    */
/*                                                                   */
/*    Miscellaneous                                                  */
/*     - InitVar                                                     */
/*     - UpdtVar                                                     */
/*     - FreeVar                                                     */
/*     - SetRxVar                                                    */
/*     - MyDiffTime                                                  */
/*     - Str2Num                                                     */
/*                                                                   */
/*********************************************************************/

APIRET CallDevice( HFILE hcomPort,
                   ULONG ulCategory,
                   ULONG ulFunction,
                   PVOID pParameters,
                   PVOID pDataPacket )
{
   APIRET       rc;                    /* Function return code       */
   PVOID        pParmPkt;              /* DevIOCtl parm packet ptr   */
   PVOID        pDataPkt;              /* DevIOCtl data packet ptr   */
   ULONG        ulParmLen;             /* DevIOCtl parm pack length  */
   ULONG        ulDataLen;             /* DevIOCtl data pack length  */
   PULONG       pulParmLen;            /* DevIOCtl parm pack len ptr */
   PULONG       pulDataLen;            /* DevIOCtl data pack len ptr */
   ULONG        ulSaveLen;             /* Save the data pack length  */
   BYTE         bGenParm;              /* General DevIOCtl parm pack */
   BYTE         bGenData;              /* General DevIOCtl data pack */

   rc = NO_ERROR;

   ulParmLen  = 0;
   ulDataLen  = 0;
   pulParmLen = &ulParmLen;
   pulDataLen = &ulDataLen;
   ulSaveLen  = 0;

   switch( ulCategory )
   {
      case IOCTL_ASYNC:
           pParmPkt = pParameters;
           pDataPkt = pDataPacket;
           switch( ulFunction )
           {
              case ASYNC_GETBAUDRATE:
                   ulDataLen = GETBAUDRATE_SIZE;
                   break;
              case ASYNC_SETBAUDRATE:
                   ulParmLen = SETBAUDRATE_SIZE;
                   break;

              case ASYNC_GETLINECTRL:
                   ulDataLen = GETLINECONTROL_SIZE;
                   break;
              case ASYNC_SETLINECTRL:
                   ulParmLen = SETLINECONTROL_SIZE;
                   break;

              case ASYNC_GETEXTDRATE:
                   ulDataLen = GETEXTDRATE_SIZE;
                   break;
              case ASYNC_SETEXTDRATE:
                   ulParmLen = SETEXTDRATE_SIZE;
                   break;

              case ASYNC_TRANSMITIMM:
                   ulParmLen = TRANSMITIMM_SIZE;
                   break;

              case ASYNC_SETBREAKON:
                   ulDataLen = COMERROR_SIZE;
                   break;
              case ASYNC_SETBREAKOFF:
                   ulDataLen = COMERROR_SIZE;
                   break;

              case ASYNC_SETMODEMCTRL:
                   ulParmLen = MODEMSTATUS_SIZE;
                   ulDataLen = COMERROR_SIZE;
                   break;

              case ASYNC_STARTTRANSMIT:
                   break;
              case ASYNC_STOPTRANSMIT:
                   break;

              case ASYNC_GETDCBINFO:
                   ulDataLen = GETDCBINFO_SIZE;
                   break;
              case ASYNC_SETDCBINFO:
                   ulParmLen = SETDCBINFO_SIZE;
                   break;

              case ASYNC_GETENHPARM:
                   ulDataLen = GETENHPARM_SIZE;
                   break;
              case ASYNC_SETENHPARM:
                   ulParmLen = SETENHPARM_SIZE;
                   break;

              case ASYNC_GETCOMMSTATUS:
                   ulDataLen = GETCOMMSTATUS_SIZE;
                   break;

              case ASYNC_GETLINESTATUS:
                   ulDataLen = GETLINESTATUS_SIZE;
                   break;

              case ASYNC_GETMODEMINPUT:
                   ulDataLen = GETMODEMIO_SIZE;
                   break;
              case ASYNC_GETMODEMOUTPUT:
                   ulDataLen = GETMODEMIO_SIZE;
                   break;

              case ASYNC_GETINQUECOUNT:
                   ulDataLen = GETIOQUEUE_SIZE;
                   break;
              case ASYNC_GETOUTQUECOUNT:
                   ulDataLen = GETIOQUEUE_SIZE;
                   break;

              case ASYNC_GETCOMMERROR:
                   ulDataLen = COMERROR_SIZE;
                   break;

              case ASYNC_GETCOMMEVENT:
                   ulDataLen = GETCOMMEVENT_SIZE;
                   break;

              default:
                   rc = ERROR_GEN_FAILURE;
                   break;
           }
           break;

      case IOCTL_GENERAL:
           pParmPkt = &bGenParm;
           pDataPkt = &bGenData;
           switch( ulFunction )
           {
              case DEV_FLUSHINPUT:
                   ulParmLen = DEVFLUSHPRM_SIZE;
                   ulDataLen = DEVFLUSHDAT_SIZE;
                   break;
              case DEV_FLUSHOUTPUT:
                   ulParmLen = DEVFLUSHPRM_SIZE;
                   ulDataLen = DEVFLUSHDAT_SIZE;
                   break;
              default:
                   rc = ERROR_GEN_FAILURE;
                   break;
           }
           break;

      default:
           rc = ERROR_GEN_FAILURE;
           break;
   }

   if ( rc == NO_ERROR ) {

      ulSaveLen = ulDataLen;

      if (ulParmLen == 0) {
         pParmPkt = NULL;
         pulParmLen = NULL;
      }

      if (ulDataLen == 0) {
         pDataPkt = NULL;
         pulDataLen = NULL;
      }

      if (ulDataLen > 0) {
         memset( pDataPkt, (BYTE)0, ulDataLen );       /* zeroise it */
      }

      rc = DosDevIOCtl( hcomPort,                          /* HFILE  */
                        ulCategory,                        /* ULONG  */
                        ulFunction,                        /* ULONG  */
                        pParmPkt,                          /* PVOID  */
                        ulParmLen,                         /* ULONG  */
                        pulParmLen,                        /* PULONG */
                        pDataPkt,                          /* PVOID  */
                        ulDataLen,                         /* ULONG  */
                        pulDataLen );                      /* PULONG */

      if ( rc == NO_ERROR ) {
         if ( ulDataLen != ulSaveLen ) {
            rc = ERROR_GEN_FAILURE;    /* Didn't return enough data  */
         }
      }

      if ( rc != NO_ERROR ) {
         fprintf( stdout,
                  "\nDosDevIOCtl(0x%4.4lX,0x%4.4lX) failed with RC=%d.",
                  ulCategory,
                  ulFunction,
                  rc );
      }
   }

   return( rc );                       /* Return DosDevIOCtl result  */
} /* =============================================================== */

APIRET ModemInput( PHFILE phcomPort, BYTE bSignal )
{
   BYTE         bComInput;             /* Modem Input Signals        */
   APIRET       rc;                    /* 0 if active, -2 if not     */
   rc = CallDevice( *phcomPort, IOCTL_ASYNC, ASYNC_GETMODEMINPUT, NULL, &bComInput );
   if ( rc == NO_ERROR )
      if ( !(bComInput & bSignal) )
         rc = RXASYNC_NO_SIGNAL;
   return( rc );                       /* Return DosDevIOCtl result  */
} /* =============================================================== */

APIRET ModemOutput( PHFILE phcomPort, BYTE bSignal )
{
   BYTE         bComOutput;            /* Modem Output Signals       */
   APIRET       rc;                    /* 0 if active, -2 if not     */
   rc = CallDevice( *phcomPort, IOCTL_ASYNC, ASYNC_GETMODEMOUTPUT, NULL, &bComOutput );
   if ( rc == NO_ERROR )
      if ( !(bComOutput & bSignal) )
         rc = RXASYNC_NO_SIGNAL;
   return( rc );                       /* Return DosDevIOCtl result  */
} /* =============================================================== */

VOID PrtDcbInfo( PDCBINFO2 pdcbPortInfo )
{
   fprintf( stdout, "\nCurrent DCBINFO settings:" );
   fprintf( stdout, "\n" );
   fprintf( stdout, "\nWrite Timeout  =>%u<",      pdcbPortInfo->usWriteTimeout );
   fprintf( stdout, "\nRead Timeout   =>%u<",      pdcbPortInfo->usReadTimeout );
   fprintf( stdout, "\nHand Shake     =>0x%2.2X<", pdcbPortInfo->fbCtlHndShake );
   fprintf( stdout, "\nFlow Replace   =>0x%2.2X<", pdcbPortInfo->fbFlowReplace );
   fprintf( stdout, "\nTimeout        =>0x%2.2X<", pdcbPortInfo->fbTimeout );
   fprintf( stdout, "\nError Replace  =>0x%2.2X<", pdcbPortInfo->bErrorReplacementChar );
   fprintf( stdout, "\nBreak Replace  =>0x%2.2X<", pdcbPortInfo->bBreakReplacementChar );
   fprintf( stdout, "\nXON Character  =>0x%2.2X<", pdcbPortInfo->bXONChar );
   fprintf( stdout, "\nXOFF Character =>0x%2.2X<", pdcbPortInfo->bXOFFChar );
   fprintf( stdout, "\n" );
   return;
} /* =============================================================== */

BOOL FileExists( PSZ FileName )
{
   /******************************************************************/
   /*                                                                */
   /* Note:                                                          */
   /*   It is normally best to return TRUE (exists) if the find test */
   /*   fails for some unknown reason.  Thus any operation expecting */
   /*   the file to exist will be performed and that will then       */
   /*   fail/suceed.  If one was to return FALSE then operations     */
   /*   expecting it not to exist ie.  create/copy etc.  could       */
   /*   possibly overwrite a file which does in fact exist.          */
   /*                                                                */
   /******************************************************************/
   HDIR         FindHandle;
   FILEFINDBUF  FileInfo;
   ULONG        FindCount;
   APIRET       rc;
   BOOL         Exists;

   FindHandle = HDIR_CREATE;
   FindCount  = 1;
   Exists     = TRUE;

   rc = DosFindFirst( (PSZ)    FileName,
                      (PHDIR)  &FindHandle,
                      (ULONG)  FILE_NORMAL,
                      (PVOID)  &FileInfo,
                      (ULONG)  sizeof(FileInfo),
                      (PULONG) &FindCount,
                      (ULONG)  FIL_STANDARD );

   if ( rc == NO_ERROR ) {

      rc = DosFindClose( FindHandle );

      if ( rc != NO_ERROR ) {

         fprintf( stdout, "\nDosFindClose() failed with RC=%d.", rc );
      }

   } else {

      if ( rc == ERROR_NO_MORE_FILES ) {

         Exists = FALSE;

      } else {

         fprintf( stdout, "\nDosFindFirst('%s') failed with RC=%d.", FileName, rc );
      }
   }

   return( Exists );
} /* =============================================================== */

BOOL FileDelete( PSZ FileName )
{
   /******************************************************************/
   /*                                                                */
   /* Note:                                                          */
   /*   If it didn't exist then it may as well have been deleted.    */
   /*                                                                */
   /******************************************************************/
   APIRET       rc;
   BOOL         Deleted;

   Deleted = FALSE;

   rc = DosDelete( FileName );

   if ( rc == NO_ERROR ) {

      Deleted = TRUE;

   } else {

      if ( rc == ERROR_FILE_NOT_FOUND ) {

         Deleted = TRUE;

      } else {

         fprintf( stdout, "\nDosDelete('%s') failed with RC=%d.", FileName, rc );

      }
   }

   return( Deleted );
} /* =============================================================== */

BOOL FileCopy( PSZ FromName, PSZ ToName )
{
   APIRET       rc;
   BOOL         Copied;

   Copied = FALSE;

   rc = DosCopy( FromName,
                 ToName,
                 (ULONG) DCPY_EXISTING );

   if ( rc == NO_ERROR ) {

      Copied = TRUE;

   } else {

      if ( rc != ERROR_FILE_NOT_FOUND ) {

         fprintf( stdout, "\nDosCopy('%s', '%s') failed with RC=%d.", FromName, ToName, rc );

      }
   }

   return( Copied );
} /* =============================================================== */

BOOL FileRename( PSZ FromName, PSZ ToName )
{
   APIRET       rc;
   BOOL         Renamed;

   Renamed = FALSE;

   rc = DosMove( FromName, ToName );

   if ( rc == NO_ERROR ) {

      Renamed = TRUE;

   } else {

      if ( rc != ERROR_FILE_NOT_FOUND ) {

         fprintf( stdout, "\nDosMove('%s', '%s') failed with RC=%d.", FromName, ToName, rc );

      }
   }

   return( Renamed );
} /* =============================================================== */

BOOL FileMove( PSZ FromName, PSZ ToName )
{
   /******************************************************************/
   /*                                                                */
   /* Note:                                                          */
   /*   Doing it this way means we can move files across drives.     */
   /*                                                                */
   /******************************************************************/
   APIRET       rc;
   BOOL         Moved;

   Moved = FALSE;

   if ( !FileExists( ToName ) ) {

      if ( FileCopy( FromName, ToName ) ) {

         if ( FileDelete( FromName ) ) {
            Moved = TRUE;
         } else {
            FileDelete( ToName );
         }
      }
   }

   return( Moved );
} /* =============================================================== */

/*********************************************************************/
/* Function:  InitVar( &rexxvar, &nextvar, varname )                 */
/*                                                                   */
/* Purpose:   Initialises Rexx variable with the specified name in   */
/*            preparation for querying the variable pool.            */
/*********************************************************************/

VOID InitVar(
        PSHVBLOCK  RexxVar,
        PSHVBLOCK  NextPtr,
        PSZ        VarName )
{
   RexxVar->shvcode            = RXSHV_SYFET;
   RexxVar->shvret             = (UCHAR) 0;
   RexxVar->shvnext            = NextPtr;
   RexxVar->shvname.strptr     = VarName;
   RexxVar->shvname.strlength  = strlen(VarName);
   RexxVar->shvnamelen         = strlen(VarName);
   RexxVar->shvvalue.strptr    = NULL;
   RexxVar->shvvalue.strlength = 0;
   RexxVar->shvvaluelen        = 0;
   return;
}

/*********************************************************************/
/* Function:  UpdtVar( &rexxvar, type, &value )                      */
/*                                                                   */
/* Purpose:   Updates the Rexx variable with the specified value in  */
/*            preparation for return to the variable pool.           */
/*********************************************************************/

VOID UpdtVar(
        PSHVBLOCK  RexxVar,
        USHORT     VarType,
        PVOID      VarValue )
{
   RexxVar->shvcode            = RXSHV_SYSET;
   RexxVar->shvret             = (UCHAR) 0;
   (RexxVar->shvvalue.strptr)[0] = RXASYNC_EOSTR_CH;
   switch( VarType )
   {
      case 1:
           sprintf( RexxVar->shvvalue.strptr, "%d",    *((PBYTE)VarValue) );
           break;
      case 2:
           sprintf( RexxVar->shvvalue.strptr, "%d",    *((PUSHORT)VarValue) );
           break;
      case 3:
           sprintf( RexxVar->shvvalue.strptr, "%d",    *((PULONG)VarValue) );
           break;
      case 4:
           sprintf( RexxVar->shvvalue.strptr, "%d",    *((PHFILE)VarValue) );
           break;
      case 5:
           sprintf( RexxVar->shvvalue.strptr, "%2.2X", *((PBYTE)VarValue) );
           break;
      default:
           strcpy( RexxVar->shvvalue.strptr,           (PSZ)VarValue );
           break;
   }
   RexxVar->shvvalue.strlength = strlen( RexxVar->shvvalue.strptr );
   RexxVar->shvvaluelen        = RexxVar->shvvalue.strlength;
   return;
}

/*********************************************************************/
/* Function:  FreeVar( &rexxvar )                                    */
/*                                                                   */
/* Purpose:   Frees the memory allocated to a Rexx variable value    */
/*            which is allocated when fetching the variable from the */
/*            variable pool.                                         */
/*********************************************************************/

VOID FreeVar(
        PSHVBLOCK  RexxVar )
{
   if ( RexxVar->shvvalue.strptr != NULL )
   {
      DosFreeMem( RexxVar->shvvalue.strptr );
      RexxVar->shvvalue.strlength = 0;
      RexxVar->shvvaluelen = 0;
   }
   return;
}

/*********************************************************************/
/* Function:  rc = SetRxVar( &rexxvar, name, type, &value )          */
/*                                                                   */
/* Purpose:   Initialises rexx variable with the specified name and  */
/*            fetches its contents from the rexx variable pool then  */
/*            updates the rexx variable with the specified value and */
/*            returns it to the variable pool and finally frees the  */
/*            memory which was allocated by the rexx variable pool   */
/*            interface when fetching the variable from the pool.    */
/*                                                                   */
/* Notes:     Currently only handles normal (256byte) values.        */
/*********************************************************************/

APIRET SetRxVar(
          PSHVBLOCK RxVarPtr,
          PSZ       RxVarName,
          USHORT    RxVarType,
          PVOID     RxVarValue
       )
{
   APIRET   rc;                        /* Function return code       */
   APIRET   rxrc;                      /* RexxVariablePool return    */

   rc   = NO_ERROR;
   rxrc = RXSHV_OK;

   /* ---------------------------------------- */
   /* Initialise Local Rexx variable for fetch */
   /* ---------------------------------------- */
   RxVarPtr->shvcode            = RXSHV_SYFET;
   RxVarPtr->shvret             = (UCHAR) 0;
   RxVarPtr->shvnext            = NULL;             /* one at a time */
   RxVarPtr->shvname.strptr     = RxVarName;
   RxVarPtr->shvname.strlength  = strlen(RxVarName);
   RxVarPtr->shvnamelen         = strlen(RxVarName);
   RxVarPtr->shvvalue.strptr    = NULL;
   RxVarPtr->shvvalue.strlength = 0;
   RxVarPtr->shvvaluelen        = 0;

   /* ------------------------------------------ */
   /* Fetch Rexx variable from the Variable Pool */
   /* ------------------------------------------ */
   rxrc = RexxVariablePool( RxVarPtr );
   if ( rxrc == RXSHV_MEMFL ) {
      rc = ERROR_NOT_ENOUGH_MEMORY;
   } else if ( rxrc != RXSHV_OK )  {
      rc = ERROR_INVALID_PARAMETER;
   } else {

      /* ----------------------------------- */
      /* Alter contents of the Rexx variable */
      /* ----------------------------------- */
      RxVarPtr->shvcode              = RXSHV_SYSET;
      RxVarPtr->shvret               = (UCHAR) 0;
      (RxVarPtr->shvvalue.strptr)[0] = RXASYNC_EOSTR_CH;
      switch( RxVarType )
      {
         case 1:
              sprintf( RxVarPtr->shvvalue.strptr, "%d",    *((PBYTE)RxVarValue) );
              break;
         case 2:
              sprintf( RxVarPtr->shvvalue.strptr, "%d",    *((PUSHORT)RxVarValue) );
              break;
         case 3:
              sprintf( RxVarPtr->shvvalue.strptr, "%d",    *((PULONG)RxVarValue) );
              break;
         case 4:
              sprintf( RxVarPtr->shvvalue.strptr, "%d",    *((PHFILE)RxVarValue) );
              break;
         case 5:
              sprintf( RxVarPtr->shvvalue.strptr, "%2.2X", *((PBYTE)RxVarValue) );
              break;
         case 6:
              sprintf( RxVarPtr->shvvalue.strptr, "%4.4X", *((PUSHORT)RxVarValue) );
              break;
         case 7:
              sprintf( RxVarPtr->shvvalue.strptr, "%8.8X", *((PULONG)RxVarValue) );
              break;
         default:
              strcpy( RxVarPtr->shvvalue.strptr,           (PSZ)RxVarValue );
              break;
      }
      RxVarPtr->shvvalue.strlength = strlen( RxVarPtr->shvvalue.strptr );
      RxVarPtr->shvvaluelen        = RxVarPtr->shvvalue.strlength;

      /* ----------------------------------------- */
      /* Update Rexx variable to the Variable Pool */
      /* ----------------------------------------- */
      rxrc = RexxVariablePool( RxVarPtr );
      if ( rxrc != RXSHV_OK ) {
         rc = ERROR_INVALID_PARAMETER;
      }
   }

   /* -------------------------------------------- */
   /* Free Local Rexx variable memory if allocated */
   /* -------------------------------------------- */
   if ( RxVarPtr->shvvalue.strptr != NULL )
   {
      DosFreeMem( RxVarPtr->shvvalue.strptr );
      RxVarPtr->shvvalue.strlength = 0;
      RxVarPtr->shvvaluelen = 0;
   }

   return( rc );
}

/*********************************************************************/
/* Function:  MyDiffTime( &now, &then )                              */
/*                                                                   */
/* Purpose:   Calculates the difference in hundredths of a second    */
/*            between two specified times. The maximum difference    */
/*            permitted is 23hrs 59min 99/100ths.                    */
/*                                                                   */
/* Return:    difference in milliseconds.                            */
/*********************************************************************/

LONG MyDiffTime(
        PDATETIME Tnow,
        PDATETIME Tthen )
{
   LONG  Tdiff;
   SHORT Tdays, Thours, Tmins, Tsecs, Thunds;

   Tdays   = Tnow->weekday    - Tthen->weekday;
   Thours  = Tnow->hours      - Tthen->hours;
   Tmins   = Tnow->minutes    - Tthen->minutes;
   Tsecs   = Tnow->seconds    - Tthen->seconds;
   Thunds  = Tnow->hundredths - Tthen->hundredths;

   if (Thunds  < 0) { Thunds  = Thunds  + 100; Tsecs   = Tsecs   - 1; }
   if (Tsecs   < 0) { Tsecs   = Tsecs   + 60;  Tmins   = Tmins   - 1; }
   if (Tmins   < 0) { Tmins   = Tmins   + 60;  Thours  = Thours  - 1; }
   if (Thours  < 0) { Thours  = Thours  + 24;  Tdays   = Tdays   - 1; }
   if (Tdays   < 0) { Tdays   = Tdays   + 7; }

   if (Tdays != 0)
      Tdiff = -1;    /* do not handle, too long a period */
   else
      Tdiff = Thunds
            + (100*Tsecs)
            + (100*60*Tmins)
            + (100*60*60*Thours);

   return( Tdiff*10 );                 /* return as milliseconds     */
}

/*********************************************************************/
/* Function:  Str2Num( numstr, numptr, numtype )                     */
/*                                                                   */
/* Purpose:   Validates and converts an ASCII-Z string from string   */
/*            form to designated number. Returns FALSE if the number */
/*            is not valid, TRUE if the number was successfully      */
/*            converted.                                             */
/*                                                                   */
/* Return:    TRUE - Good number converted                           */
/*            FALSE - Invalid number supplied.                       */
/*********************************************************************/
BOOL Str2Num( PSZ NumStr, PVOID pNumber, USHORT NumType )
{
   ULONG  Tnumber;
   ULONG  Tmax;
   USHORT Trad;
   USHORT Tlen;
   USHORT Tind;
   USHORT Tpos;
   SHORT  Tsgn;
   CHAR   Tdig[2];

   Tnumber = 0;
   Tmax    = 0;
   Trad    = 0;
   Tlen    = 0;
   Tind    = 0;
   Tpos    = 0;
   Tsgn    = 0;
   Tdig[0] = '\0';
   Tdig[1] = '\0';

   switch( NumType )
   {
      case STR2NUM_BIN2BYTE:   /* binary  to byte   */ Tlen =  8; Trad =  2; Tmax = MAX_BYTE_DEC;   break;
      case STR2NUM_BIN2SHORT:  /* binary  to short  */ Tlen = 16; Trad =  2; Tmax = MAX_SHORT_DEC;  break;
      case STR2NUM_BIN2USHORT: /* binary  to ushort */ Tlen = 16; Trad =  2; Tmax = MAX_USHORT_DEC; break;
      case STR2NUM_BIN2LONG:   /* binary  to long   */ Tlen = 32; Trad =  2; Tmax = MAX_LONG_DEC;   break;
      case STR2NUM_BIN2ULONG:  /* binary  to ulong  */ Tlen = 32; Trad =  2; Tmax = MAX_ULONG_DEC;  break;
      case STR2NUM_HEX2BYTE:   /* hex     to byte   */ Tlen =  2; Trad = 16; Tmax = MAX_BYTE_DEC;   break;
      case STR2NUM_HEX2SHORT:  /* hex     to short  */ Tlen =  4; Trad = 16; Tmax = MAX_SHORT_DEC;  break;
      case STR2NUM_HEX2USHORT: /* hex     to ushort */ Tlen =  4; Trad = 16; Tmax = MAX_USHORT_DEC; break;
      case STR2NUM_HEX2LONG:   /* hex     to long   */ Tlen =  8; Trad = 16; Tmax = MAX_LONG_DEC;   break;
      case STR2NUM_HEX2ULONG:  /* hex     to ulong  */ Tlen =  8; Trad = 16; Tmax = MAX_ULONG_DEC;  break;
      case STR2NUM_OCT2BYTE:   /* octal   to byte   */ Tlen =  3; Trad =  8; Tmax = MAX_BYTE_DEC;   break;
      case STR2NUM_OCT2SHORT:  /* octal   to short  */ Tlen =  6; Trad =  8; Tmax = MAX_SHORT_DEC;  break;
      case STR2NUM_OCT2USHORT: /* octal   to ushort */ Tlen =  6; Trad =  8; Tmax = MAX_USHORT_DEC; break;
      case STR2NUM_OCT2LONG:   /* octal   to long   */ Tlen =  9; Trad =  8; Tmax = MAX_LONG_DEC;   break;
      case STR2NUM_OCT2ULONG:  /* octal   to ulong  */ Tlen =  9; Trad =  8; Tmax = MAX_ULONG_DEC;  break;
      case STR2NUM_DEC2BYTE:   /* decimal to byte   */ Tlen =  3; Trad = 10; Tmax = MAX_BYTE_DEC;   break;
      case STR2NUM_DEC2SHORT:  /* decimal to short  */ Tlen =  5; Trad = 10; Tmax = MAX_SHORT_DEC;  break;
      case STR2NUM_DEC2USHORT: /* decimal to ushort */ Tlen =  5; Trad = 10; Tmax = MAX_USHORT_DEC; break;
      case STR2NUM_DEC2LONG:   /* decimal to long   */ Tlen = 10; Trad = 10; Tmax = MAX_LONG_DEC;   break;
      case STR2NUM_DEC2ULONG:  /* decimal to ulong  */ Tlen = 10; Trad = 10; Tmax = MAX_ULONG_DEC;  break;
      default:
           return( FALSE );
   }

   while (isspace( *NumStr )) { NumStr++; } /* skip leading blanks */

   if (*NumStr == '+') { NumStr++; } /* skip positive sign */

   if (*NumStr == '-') {
      switch( NumType )
      {
         case STR2NUM_DEC2SHORT:
         case STR2NUM_DEC2LONG:
              /* to short or long */
              break;
         default:
              return( FALSE );
      }
      Tsgn = -1;
      NumStr++;
   } /* set negative and skip sign */

   if (strlen( NumStr ) == 0) { return( FALSE ); } /* not a number */

   Tind = 0;
   Tpos = 0;

   do {
      Tdig[0] = NumStr[0];
      Tind = strcspn( DIGIT_CHARS, Tdig );

      if ((Trad == 16) && (Tind > 15)) { Tind = Tind - 6; } /* lowercase hex adjust */

      if (Tind < Trad)
      {
         Tnumber = Tnumber*Trad + Tind;

         if (Tnumber > (Tmax - Tsgn)) { return( FALSE ); } /* exceeds bounds */

         Tpos = Tpos + 1;
         NumStr++;

      } /* else not a valid digit or end of string */

   } while ((Tpos < Tlen) && (Tind < Trad));

   while (isspace( *NumStr )) { NumStr++; }     /* skip trailing blanks */

   if (*NumStr != '\0') { return( FALSE ); }    /* not a number */

   switch( NumType )
   {
      case STR2NUM_BIN2BYTE:
      case STR2NUM_HEX2BYTE:
      case STR2NUM_OCT2BYTE:
      case STR2NUM_DEC2BYTE:
           /* to byte   */
           *((PBYTE) pNumber)   = (BYTE) Tnumber;
           break;
      case STR2NUM_BIN2USHORT:
      case STR2NUM_HEX2USHORT:
      case STR2NUM_OCT2USHORT:
      case STR2NUM_DEC2USHORT:
           /* to ushort */
           *((PUSHORT) pNumber) = (USHORT) Tnumber;
           break;
      case STR2NUM_BIN2ULONG:
      case STR2NUM_HEX2ULONG:
      case STR2NUM_OCT2ULONG:
      case STR2NUM_DEC2ULONG:
           /* to ulong  */
           *((PULONG) pNumber)  = (ULONG) Tnumber;
           break;
      case STR2NUM_BIN2SHORT:
      case STR2NUM_HEX2SHORT:
      case STR2NUM_OCT2SHORT:
      case STR2NUM_DEC2SHORT:
           /* to short  */
           if (Tsgn < 0) {
              *((PSHORT) pNumber)  = (SHORT) 0 - (SHORT) Tnumber;
           } else {
              *((PSHORT) pNumber)  = (SHORT) Tnumber;
           }
           break;
      case STR2NUM_BIN2LONG:
      case STR2NUM_HEX2LONG:
      case STR2NUM_OCT2LONG:
      case STR2NUM_DEC2LONG:
           /* to long   */
           if (Tsgn < 0) {
              *((PLONG) pNumber)   = (LONG) 0 - (LONG) Tnumber;
           } else {
              *((PLONG) pNumber)   = (LONG) Tnumber;
           }
           break;
      default:
           return( FALSE );
   }

   return( TRUE );
}

/*********************************************************************/
/* RXASYNC functions.                                                */
/*********************************************************************/

/*********************************************************************/
/*                                                                   */
/* Function:  RxAsyncLoadFuncs                                       */
/*                                                                   */
/* Purpose:   Load the functions                                     */
/*                                                                   */
/* Syntax:    call RxAsyncLoadFuncs                                  */
/*                                                                   */
/* Params:    none                                                   */
/*                                                                   */
/* Return:    NO_ERROR (0)                                           */
/*                                                                   */
/* Example:                                                          */
/*                                                                   */
/*   call RxFuncAdd 'RxAsyncLoadFuncs','RXASYNC','RxAsyncLoadFuncs'  */
/*   call RxAsyncLoadFuncs                                           */
/*   Say 'RxAsyncLoadFuncs is complete.'                             */
/*                                                                   */
/*********************************************************************/

APIRET RxAsyncLoadFuncs(
          CHAR     *name,
          ULONG    numargs,
          RXSTRING args[],
          CHAR     *queuename,
          RXSTRING *retstr
       )
{
   LONG  entries;                      /* Num of entries             */
   LONG  j;                            /* Counter                    */

   if ( numargs != 0 )                 /* Must have no arguments     */
      return( ROUTINE_INVALID );

   retstr->strlength = 0;              /* Return a null string result*/

   entries = sizeof( RxFncTable ) / sizeof( PSZ );

   for (j = 0; j < entries; j++) {
      RexxRegisterFunctionDll( RxFncTable[j],
                               RXASYNC_DLL_NAME,
                               RxFncTable[j] );
      /* note: could return RXFUNC_NOMEM */
   }

   BUILDRXDOSRET( retstr, NO_ERROR );  /* Initialise return value    */

   return( ROUTINE_VALID );            /* All ok                     */
}

/*********************************************************************/
/*                                                                   */
/* Function:  RxAsyncDropFuncs                                       */
/*                                                                   */
/* Purpose:   Unload the functions                                   */
/*                                                                   */
/* Syntax:    call RxAsyncDropFuncs                                  */
/*                                                                   */
/* Parms:     none                                                   */
/*                                                                   */
/* Return:    NO_ERROR (0)                                           */
/*                                                                   */
/* Example:                                                          */
/*                                                                   */
/*   call RxAsyncDropFuncs                                           */
/*   Say 'RxAsyncDropFuncs is complete.'                             */
/*                                                                   */
/*********************************************************************/

APIRET RxAsyncDropFuncs(
          CHAR     *name,
          ULONG    numargs,
          RXSTRING args[],
          CHAR     *queuename,
          RXSTRING *retstr
       )
{
   LONG  entries;                      /* Num of entries             */
   LONG  j;                            /* Counter                    */

   if ( numargs != 0 )                 /* Must have no arguments     */
      return( ROUTINE_INVALID );

   retstr->strlength = 0;              /* Return a null string result*/

   entries = sizeof( RxFncTable ) / sizeof( PSZ );

   for (j = 0; j < entries; j++) {
      RexxDeregisterFunction( RxFncTable[j] );
   }

   BUILDRXDOSRET( retstr, NO_ERROR );  /* Initialise return value    */

   return( ROUTINE_VALID );
}

/*********************************************************************/
/*                                                                   */
/* Function:  RxAsyncPriority                                        */
/*                                                                   */
/* Purpose:   Set Rexx process priority                              */
/*                                                                   */
/* Syntax:    rc = RxAsyncPriority( class, level )                   */
/*                                                                   */
/* Params:    class - Priority class 0..4                            */
/*                       0 = NOCHANGE                                */
/*                       1 = IDLETIME                                */
/*                       2 = REGULAR                                 */
/*                       3 = TIMECRITICAL                            */
/*                       4 = FOREGROUNDSERVER                        */
/*            level - Change current level -31..+31                  */
/*                       (normally zero)                             */
/*                                                                   */
/* Return:                                                           */
/*   Call successful - NO_ERROR (0)                                  */
/*   Call failed     - rc from DosSetPriority                        */
/*                                                                   */
/* Example:                                                          */
/*                                                                   */
/*   Crit    = 3                                                     */
/*   NormLvl = 0                                                     */
/*   rc = RxAsyncPriority( Crit, NormLvl )                           */
/*   if rc <> 0                                                      */
/*   then say 'RxAsyncPriority failed with rc=>'rc'<'                */
/*   else say 'RxAsyncPriority, Class=>'Crit'<, Level=>'NormLvl'<'   */
/*                                                                   */
/*********************************************************************/

APIRET RxAsyncPriority(
          CHAR     *name,
          ULONG    numargs,
          RXSTRING args[],
          CHAR     *queuename,
          RXSTRING *retstr
       )
{
  ULONG     class;                     /* priority class             */
  LONG      level;                     /* priority level             */
  APIRET    rc;                        /* creation return code       */

   if ( numargs != 2 )                 /* Must have two arguments    */
      return( ROUTINE_INVALID );

   if (   !RXVALIDSTRING(args[0])      /* Must specify class         */
       || !RXVALIDSTRING(args[1]) )    /* and priority level         */
      return( ROUTINE_INVALID );

   if ( !Str2Num(args[0].strptr, &class, STR2NUM_DEC2ULONG) )
      return( ROUTINE_INVALID );       /* Convert class failed       */

   if ( !Str2Num(args[1].strptr, &level, STR2NUM_DEC2LONG) )
      return( ROUTINE_INVALID );       /* Convert level failed       */

   if (   (class < PRTYC_NOCHANGE)
       || (class > PRTYC_FOREGROUNDSERVER) )
      return( ROUTINE_INVALID );       /* invalid class              */

   if (   (level < PRTYD_MINIMUM)
       || (level > PRTYD_MAXIMUM) )
      return( ROUTINE_INVALID );       /* invalid priority           */

   rc = DosSetPrty( PRTYS_THREAD,
                    class,
                    level,
                    0 );               /* current process/thread     */

   if ( rc != NO_ERROR ) {
      fprintf( stdout, "\nDosSetPrty() failed with RC=%d.", rc );
   }

   BUILDRXDOSRET( retstr, rc );        /* Return DosSetPriority rc   */

   return( ROUTINE_VALID );
}

/*********************************************************************/
/*                                                                   */
/* Function:  RxAsyncSleep                                           */
/*                                                                   */
/* Purpose:   Sleep the specified length of time                     */
/*                                                                   */
/* Syntax:    call RxAsyncSleep milliseconds                         */
/*                                                                   */
/* Params:    millisecs - length of time to sleep                    */
/*                                                                   */
/* Return:    NO_ERROR (0)                                           */
/*                                                                   */
/* Example:                                                          */
/*                                                                   */
/*   say 'Sleep for a bit 'time()                                    */
/*   call RxAsyncSleep 2000                                          */
/*   say 'Slept enough 'time()                                       */
/*                                                                   */
/*********************************************************************/

APIRET RxAsyncSleep(
          CHAR     *name,
          ULONG    numargs,
          RXSTRING args[],
          CHAR     *queuename,
          RXSTRING *retstr
       )
{
   LONG  millisecs;                    /* Time to sleep              */

   if ( numargs != 1 )                 /* Must have one argument     */
      return( ROUTINE_INVALID );

   if (   !RXVALIDSTRING(args[0]) )    /* Must specify time limit    */
      return( ROUTINE_INVALID );

   if ( !Str2Num(args[0].strptr, &millisecs, STR2NUM_DEC2LONG) )
      return( ROUTINE_INVALID );       /* Convert millisecs failed   */

   if ( millisecs < 0 )                /* Can not wait negative time */
      return( ROUTINE_INVALID );

   if ( millisecs > 0 )                /* Only if its worthwile      */
      DosSleep( millisecs );

   BUILDRXDOSRET( retstr, NO_ERROR );  /* Initialise return value    */

   return( ROUTINE_VALID );            /* All ok                     */
}

/*********************************************************************/
/*                                                                   */
/* Function:  RxAsyncGetDcbInfo                                      */
/*                                                                   */
/* Purpose:   Return current port settings                           */
/*                                                                   */
/*            Utilises IOCTL 73H.                                    */
/*                                                                   */
/* Syntax:    call RxAsyncGetDcbInfo handlevar, write_timeout_var,   */
/*                                              read_timeout_var,    */
/*                                              handshake_var,       */
/*                                              flow_replace_var,    */
/*                                              timeout_var,         */
/*                                              error_replace_var,   */
/*                                              break_replace_var,   */
/*                                              xon_var,             */
/*                                              xoff_var             */
/*                                                                   */
/* Params:    handlevar  - rexx variable with com handle             */
/*            writevar   - rexx variable to store write timeout      */
/*            readvar    - rexx variable to store read timeout       */
/*            handflgvar - rexx variable to store handshake flags 1  */
/*            flowflgvar - rexx variable to store flow-repl flags 2  */
/*            timeflgvar - rexx variable to store timeout flags 3    */
/*            errreplvar - rexx variable to store error-repl char    */
/*            brkreplvar - rexx variable to store break-repl char    */
/*            xonvar     - rexx variable to store xon char           */
/*            xoffvar    - rexx variable to store xoff char          */
/*                                                                   */
/* Return:                                                           */
/*   Call successful - NO_ERROR (0)                                  */
/*   Call failed     - rc from DosDevIOCtl                           */
/*                                                                   */
/* Example:                                                          */
/*                                                                   */
/*   rc = RxAsyncGetDcbInfo( PortHandle, 'wlim','rlim','flags1',     */
/*                           'flags2','flags3','errch','brkch',      */
/*                           'xonch','xoffch' )                      */
/*   if rc <> 0                                                      */
/*      then say 'RxAsyncGetDcbInfo failed with rc=>'rc'<'           */
/*      else say 'RxAsyncGetDcbInfo returned okay.'                  */
/*                                                                   */
/*********************************************************************/

APIRET RxAsyncGetDcbInfo(
          CHAR     *name,
          ULONG    numargs,
          RXSTRING args[],
          CHAR     *queuename,
          RXSTRING *retstr
       )
{
   APIRET   rc;                        /* Function return code       */
   APIRET   rxrc;                      /* RexxVariablePool return    */
   HFILE    hcomPort;                  /* Communications port handle */
   DCBINFO2 dcbPortInfo;               /* Device control block       */
   SHVBLOCK WlimVar;                   /* Rexx var for write timeout */
   SHVBLOCK RlimVar;                   /* Rexx var for read timeout  */
   SHVBLOCK Flg1Var;                   /* Rexx var for flags 1       */
   SHVBLOCK Flg2Var;                   /* Rexx var for flags 2       */
   SHVBLOCK Flg3Var;                   /* Rexx var for flags 3       */
   SHVBLOCK ErrcVar;                   /* Rexx var for err repl char */
   SHVBLOCK BrkcVar;                   /* Rexx var for brk repl char */
   SHVBLOCK XoncVar;                   /* Rexx var for xon char      */
   SHVBLOCK XoffVar;                   /* Rexx var for xoff char     */

   if ( numargs != 10 )                /* Must have ten arguments    */
      return( ROUTINE_INVALID );

   if (   !RXVALIDSTRING(args[0])      /* Must specify port          */
       || !RXVALIDSTRING(args[1])      /* and write timeout var name */
       || !RXVALIDSTRING(args[2])      /* and read timeout var name  */
       || !RXVALIDSTRING(args[3])      /* and flags 1 var name       */
       || !RXVALIDSTRING(args[4])      /* and flags 2 var name       */
       || !RXVALIDSTRING(args[5])      /* and flags 3 var name       */
       || !RXVALIDSTRING(args[6])      /* and error replace var name */
       || !RXVALIDSTRING(args[7])      /* and break replace var name */
       || !RXVALIDSTRING(args[8])      /* and xon character var name */
       || !RXVALIDSTRING(args[9]) )    /* and xoff character var name*/
      return( ROUTINE_INVALID );

   if ( !Str2Num(args[0].strptr, &hcomPort, STR2NUM_DEC2ULONG) )
      return( ROUTINE_INVALID );       /* Convert handle failed      */

   rc = CallDevice( hcomPort, IOCTL_ASYNC, ASYNC_GETDCBINFO, NULL, &dcbPortInfo );
   if ( rc == NO_ERROR ) {

      /* fetch rexx variables details (rexx mallocs, we free) */

      InitVar( &WlimVar, &RlimVar, args[1].strptr );
      InitVar( &RlimVar, &Flg1Var, args[2].strptr );
      InitVar( &Flg1Var, &Flg2Var, args[3].strptr );
      InitVar( &Flg2Var, &Flg3Var, args[4].strptr );
      InitVar( &Flg3Var, &ErrcVar, args[5].strptr );
      InitVar( &ErrcVar, &BrkcVar, args[6].strptr );
      InitVar( &BrkcVar, &XoncVar, args[7].strptr );
      InitVar( &XoncVar, &XoffVar, args[8].strptr );
      InitVar( &XoffVar, NULL,     args[9].strptr );

      rxrc = RexxVariablePool( &WlimVar ); /* GET the rexx variables */

      if ( rxrc == RXSHV_MEMFL ) {     /* get vars failed no memory  */
         rc = ERROR_NOT_ENOUGH_MEMORY;
      } else if ( rxrc != RXSHV_OK ) { /* get vars failed            */
         rc = ERROR_INVALID_PARAMETER;
      } else {

         /* set rexx variable details with retrieved values */

         UpdtVar( &WlimVar, 2, &(dcbPortInfo.usWriteTimeout) );
         UpdtVar( &RlimVar, 2, &(dcbPortInfo.usReadTimeout) );
         UpdtVar( &Flg1Var, 5, &(dcbPortInfo.fbCtlHndShake) );
         UpdtVar( &Flg2Var, 5, &(dcbPortInfo.fbFlowReplace) );
         UpdtVar( &Flg3Var, 5, &(dcbPortInfo.fbTimeout) );
         UpdtVar( &ErrcVar, 5, &(dcbPortInfo.bErrorReplacementChar) );
         UpdtVar( &BrkcVar, 5, &(dcbPortInfo.bBreakReplacementChar) );
         UpdtVar( &XoncVar, 5, &(dcbPortInfo.bXONChar) );
         UpdtVar( &XoffVar, 5, &(dcbPortInfo.bXOFFChar) );

         rxrc = RexxVariablePool( &WlimVar ); /* SET the rexx variables */

         if ( rxrc != RXSHV_OK ) {     /* set rexx variables failed  */
            rc = ERROR_INVALID_PARAMETER;
         }
      }

      FreeVar( &WlimVar );             /* Cleanup                    */
      FreeVar( &RlimVar );
      FreeVar( &Flg1Var );
      FreeVar( &Flg2Var );
      FreeVar( &Flg3Var );
      FreeVar( &ErrcVar );
      FreeVar( &BrkcVar );
      FreeVar( &XoncVar );
      FreeVar( &XoffVar );
   }

   BUILDRXDOSRET( retstr, rc );        /* Return DosDevIOCtl result  */

   return( ROUTINE_VALID );
}

/*********************************************************************/
/*                                                                   */
/* Function:  RxAsyncSetDcbInfo                                      */
/*                                                                   */
/* Purpose:   Update current port settings                           */
/*                                                                   */
/*            Utilises IOCTLs 53H and 73H.                           */
/*                                                                   */
/* Syntax:    call RxAsyncSetDcbInfo port,                           */
/*                                   write_timeout,                  */
/*                                   read_timeout,                   */
/*                                   handshake,                      */
/*                                   flow_replace,                   */
/*                                   timeout,                        */
/*                                   error_replace,                  */
/*                                   break_replace,                  */
/*                                   xon,                            */
/*                                   xoff                            */
/*                                                                   */
/* Params:    port       - com port handle                           */
/*            wlimit     - write timeout                             */
/*            rlimit     - read timeout                              */
/*            flags1     - handshake flags 1                         */
/*            flags2     - flow-replace flags 2                      */
/*            flags3     - timeout flags 3                           */
/*            err_repl   - error-replace char                        */
/*            brk_repl   - break-replace char                        */
/*            xon        - xon char                                  */
/*            xoff       - xoff char                                 */
/*                                                                   */
/* Return:                                                           */
/*   Call successful - NO_ERROR (0)                                  */
/*   Call failed     - rc from DosDevIOCtl                           */
/*                                                                   */
/* Notes:                                                            */
/*   Parameters other than the PortHandle are optional in that if    */
/*   specified as null items they will not cause the existing        */
/*   settings to be alterred.                                        */
/*                                                                   */
/* Example:                                                          */
/*                                                                   */
/*   rc = RxAsyncSetDcbInfo( PortHandle,                             */
/*                           100,                                    */
/*                           50,                                     */
/*                           '01001001',                             */
/*                           '10100000',                             */
/*                           '11010010',                             */
/*                           '00',                                   */
/*                           '00',                                   */
/*                           '11',                                   */
/*                           '13' )                                  */
/*   if rc <> 0                                                      */
/*      then say 'RxAsyncSetDcbInfo failed with rc=>'rc'<'           */
/*      else say 'RxAsyncSetDcbInfo returned okay.'                  */
/*                                                                   */
/*********************************************************************/

APIRET RxAsyncSetDcbInfo(
          CHAR     *name,
          ULONG    numargs,
          RXSTRING args[],
          CHAR     *queuename,
          RXSTRING *retstr
       )
{
   APIRET   rc;                        /* Function return code       */
   HFILE    hcomPort;                  /* Communications port handle */
   DCBINFO2 dcbPortInfo;               /* Device control block       */

   if ( numargs != 10 )                /* Must have ten arguments    */
      return( ROUTINE_INVALID );

   if ( !RXVALIDSTRING(args[0]) )      /* Must specify port          */
      return( ROUTINE_INVALID );

   if ( !Str2Num(args[0].strptr, &hcomPort, STR2NUM_DEC2ULONG) )
      return( ROUTINE_INVALID );       /* Convert handle failed      */

   rc = CallDevice( hcomPort, IOCTL_ASYNC, ASYNC_GETDCBINFO, NULL, &dcbPortInfo );
   if ( rc == NO_ERROR ) {

      if ( RXVALIDSTRING(args[1]) )    /* If specify write timeout   */
         if ( !Str2Num(args[1].strptr, &(dcbPortInfo.usWriteTimeout), STR2NUM_DEC2USHORT) )
            return( ROUTINE_INVALID );

      if ( RXVALIDSTRING(args[2]) )    /* If specify read timeout    */
         if ( !Str2Num(args[2].strptr, &(dcbPortInfo.usReadTimeout), STR2NUM_DEC2USHORT) )
            return( ROUTINE_INVALID );

      if ( RXVALIDSTRING(args[3]) )    /* If specify flags 1         */
         if ( !Str2Num(args[3].strptr, &(dcbPortInfo.fbCtlHndShake), STR2NUM_BIN2BYTE) )
            return( ROUTINE_INVALID );

      if ( RXVALIDSTRING(args[4]) )    /* If specify flags 2         */
         if ( !Str2Num(args[4].strptr, &(dcbPortInfo.fbFlowReplace), STR2NUM_BIN2BYTE) )
            return( ROUTINE_INVALID );

      if ( RXVALIDSTRING(args[5]) )    /* If specify flags 3         */
         if ( !Str2Num(args[5].strptr, &(dcbPortInfo.fbTimeout), STR2NUM_BIN2BYTE) )
            return( ROUTINE_INVALID );

      if ( RXVALIDSTRING(args[6]) )    /* If specify error replace   */
         if ( !Str2Num(args[6].strptr, &(dcbPortInfo.bErrorReplacementChar), STR2NUM_HEX2BYTE) )
            return( ROUTINE_INVALID );

      if ( RXVALIDSTRING(args[7]) )    /* If specify break replace   */
         if ( !Str2Num(args[7].strptr, &(dcbPortInfo.bBreakReplacementChar), STR2NUM_HEX2BYTE) )
            return( ROUTINE_INVALID );

      if ( RXVALIDSTRING(args[8]) )    /* If specify xon character   */
         if ( !Str2Num(args[8].strptr, &(dcbPortInfo.bXONChar), STR2NUM_HEX2BYTE) )
            return( ROUTINE_INVALID );

      if ( RXVALIDSTRING(args[9]) )    /* If specify xoff character  */
         if ( !Str2Num(args[9].strptr, &(dcbPortInfo.bXOFFChar), STR2NUM_HEX2BYTE) )
            return( ROUTINE_INVALID );

      rc = CallDevice( hcomPort, IOCTL_ASYNC, ASYNC_SETDCBINFO, &dcbPortInfo, NULL );
   }

   BUILDRXDOSRET( retstr, rc );        /* Return DosDevIOCtl result  */

   return( ROUTINE_VALID );
}

/*********************************************************************/
/*                                                                   */
/* Function:  RxAsyncGetEnhParm                                      */
/*                                                                   */
/* Purpose:   Return current settings                                */
/*                                                                   */
/*            Utilises IOCTL 74H.                                    */
/*                                                                   */
/* Syntax:    call RxAsyncGetEnhParm handlevar, enhparmvar           */
/*                                                                   */
/* Params:    handlevar  - rexx variable with com handle             */
/*            enhparmvar - rexx variable to store parameter value    */
/*                                                                   */
/* Return:                                                           */
/*   Call successful - NO_ERROR (0)                                  */
/*   Call failed     - rc from DosDevIOCtl                           */
/*                                                                   */
/* Example:                                                          */
/*                                                                   */
/*   rc = RxAsyncGetEnhParm( PortHandle, 'EnhParm' )                 */
/*   if rc <> 0                                                      */
/*      then say 'RxAsyncGetEnhParm failed with rc=>'rc'<'           */
/*      else say 'RxAsyncGetEnhParm returned okay.'                  */
/*                                                                   */
/*********************************************************************/

APIRET RxAsyncGetEnhParm(
          CHAR     *name,
          ULONG    numargs,
          RXSTRING args[],
          CHAR     *queuename,
          RXSTRING *retstr
       )
{
   APIRET   rc;                        /* Function return code       */
   HFILE    hcomPort;                  /* Communications port handle */
   ENHPARM  eparmParms;                /* Enhanced Mode Parameters   */
   SHVBLOCK ParmVar;                   /* Rexx var for enh parameters*/

   if ( numargs != 2 )                 /* Must have two arguments    */
      return( ROUTINE_INVALID );

   if (   !RXVALIDSTRING(args[0])      /* Must specify port          */
       || !RXVALIDSTRING(args[1]) )    /* and enh parameters var name*/
      return( ROUTINE_INVALID );

   if ( !Str2Num(args[0].strptr, &hcomPort, STR2NUM_DEC2ULONG) )
      return( ROUTINE_INVALID );       /* Convert handle failed      */

   rc = CallDevice( hcomPort, IOCTL_ASYNC, ASYNC_GETENHPARM, NULL, &eparmParms );
   if ( rc == NO_ERROR ) {

      rc = SetRxVar( &ParmVar, args[1].strptr, 5, &(eparmParms.bEnhancedFlags) );
   }

   BUILDRXDOSRET( retstr, rc );        /* Return DosDevIOCtl result  */

   return( ROUTINE_VALID );
}

/*********************************************************************/
/*                                                                   */
/* Function:  RxAsyncSetEnhParm                                      */
/*                                                                   */
/* Purpose:   Update current settings                                */
/*                                                                   */
/*            Utilises IOCTLs 54H and 74H.                           */
/*                                                                   */
/* Syntax:    call RxAsyncSetEnhParm handlevar, enhparmvar           */
/*                                                                   */
/* Params:    handlevar  - rexx variable with com handle             */
/*            enhparmvar - rexx variable with parameter settings     */
/*                                                                   */
/* Return:                                                           */
/*   Call successful - NO_ERROR (0)                                  */
/*   Call failed     - rc from DosDevIOCtl                           */
/*                                                                   */
/* Example:                                                          */
/*                                                                   */
/*   rc = RxAsyncSetEnhParm( PortHandle, EnhParm )                   */
/*   if rc <> 0                                                      */
/*      then say 'RxAsyncSetEnhParm failed with rc=>'rc'<'           */
/*      else say 'RxAsyncSetEnhParm returned okay.'                  */
/*                                                                   */
/*********************************************************************/

APIRET RxAsyncSetEnhParm(
          CHAR     *name,
          ULONG    numargs,
          RXSTRING args[],
          CHAR     *queuename,
          RXSTRING *retstr
       )
{
   APIRET   rc;                        /* Function return code       */
   HFILE    hcomPort;                  /* Communications port handle */
   ENHPARM  eparmParms;                /* Enhanced Mode Parameters   */
   SHVBLOCK ParmVar;                   /* Rexx var for enh parameters*/

   if ( numargs != 2 )                 /* Must have two arguments    */
      return( ROUTINE_INVALID );

   if (   !RXVALIDSTRING(args[0])      /* Must specify port          */
       || !RXVALIDSTRING(args[1]) )    /* and enh parameters         */
      return( ROUTINE_INVALID );

   if ( !Str2Num(args[0].strptr, &hcomPort, STR2NUM_DEC2ULONG) )
      return( ROUTINE_INVALID );       /* Convert handle failed      */

   rc = CallDevice( hcomPort, IOCTL_ASYNC, ASYNC_GETENHPARM, NULL, &eparmParms );
   if ( rc == NO_ERROR ) {

      if ( !(eparmParms.bEnhancedFlags & 0x01 ) ) {
         /* don't allow setting, enhanced mode is not supported */
      } else {

         if ( !Str2Num(args[1].strptr, &(eparmParms.bEnhancedFlags), STR2NUM_BIN2BYTE) )
            return( ROUTINE_INVALID );

         rc = CallDevice( hcomPort, IOCTL_ASYNC, ASYNC_SETENHPARM, NULL, &eparmParms );
      }
   }

   BUILDRXDOSRET( retstr, rc );        /* Return DosDevIOCtl result  */

   return( ROUTINE_VALID );
}

/*********************************************************************/
/*                                                                   */
/* Function:  RxAsyncGetLnCtrl                                       */
/*                                                                   */
/* Purpose:   Get status of specified port                           */
/*                                                                   */
/*            Utilises IOCTLs 62H and 63H.                           */
/*                                                                   */
/* Syntax:    call RxAsyncGetLnCtrl port                             */
/*                                  'baud' 'data' 'parity' 'stop'    */
/*                                                                   */
/* Params:    port   - handle of open port                           */
/*            baud   - rexx variable to store baudrate in            */
/*            data   - rexx variable to store databits in            */
/*            parity - rexx variable to store parity in              */
/*            stop   - rexx variable to store stopbits in            */
/*                                                                   */
/* Return:                                                           */
/*   Call successful - NO_ERROR (0)                                  */
/*   Call failed     - rc from DosDevIOCtl                           */
/*                                                                   */
/* Example:                                                          */
/*                                                                   */
/*   OldBaud    = 0                                                  */
/*   OldData    = 0                                                  */
/*   OldParity  = ''                                                 */
/*   OldStop    = 0                                                  */
/*   rc = RxAsyncGetLnCtrl( PortHandle, 'OldBaud',                   */
/*                          'OldData', 'OldParity', 'OldStop' )      */
/*   if rc <> 0                                                      */
/*      then say 'RxAsyncGetLnCtrl failed with rc=>'rc'<'            */
/*      else say 'RxAsyncGetLnCtrl saved the original settings'      */
/*                                                                   */
/*   say 'Baudrate was >'OldBaud'<'                                  */
/*   say 'Databits was >'OldData'<'                                  */
/*   say 'Parity   was >'OldParity'<'                                */
/*   say 'Stopbits was >'OldStop'<'                                  */
/*                                                                   */
/*********************************************************************/

APIRET RxAsyncGetLnCtrl(
          CHAR     *name,
          ULONG    numargs,
          RXSTRING args[],
          CHAR     *queuename,
          RXSTRING *retstr
       )
{
   APIRET       rc;                    /* Function return code       */
   APIRET       rxrc;                  /* RexxVariablePool return    */
   HFILE        hcomPort;              /* Communications port handle */
   EXTDRATE     erateBaudRate;         /* Extended bitrate params    */
   LINECONTROL  lnctlLineControl;      /* Line control parameters    */
   SHVBLOCK     BaudVar;               /* Rexx variable for baudrate */
   SHVBLOCK     DataVar;               /* Rexx variable for databits */
   SHVBLOCK     PariVar;               /* Rexx variable for parity   */
   SHVBLOCK     StopVar;               /* Rexx variable for stopbits */
   PSZ          pszDataBits;
   PSZ          pszStopBits;
   PSZ          pszParity;

   if ( numargs != 5 )                 /* Must have five arguments   */
      return( ROUTINE_INVALID );

   if (   !RXVALIDSTRING(args[0])      /* Must specify port handle   */
       || !RXVALIDSTRING(args[1])      /* and baudrate var name      */
       || !RXVALIDSTRING(args[2])      /* and databits var name      */
       || !RXVALIDSTRING(args[3])      /* and parity var name        */
       || !RXVALIDSTRING(args[4]) )    /* and stopbits var name      */
      return( ROUTINE_INVALID );

   if ( !Str2Num(args[0].strptr, &hcomPort, STR2NUM_DEC2ULONG) )
      return( ROUTINE_INVALID );       /* Convert handle failed      */

   rc = CallDevice( hcomPort, IOCTL_ASYNC, ASYNC_GETEXTDRATE, NULL, &erateBaudRate );
   if ( rc == NO_ERROR ) {

      rc = CallDevice( hcomPort, IOCTL_ASYNC, ASYNC_GETLINECTRL, NULL, &lnctlLineControl );
      if ( rc == NO_ERROR ) {

         switch( lnctlLineControl.bDataBits )
         {
            case DATA_BITS_5:
                 pszDataBits = DATA_BITS_5_STR;
                 break;
            case DATA_BITS_6:
                 pszDataBits = DATA_BITS_6_STR;
                 break;
            case DATA_BITS_7:
                 pszDataBits = DATA_BITS_7_STR;
                 break;
            case DATA_BITS_8:
                 pszDataBits = DATA_BITS_8_STR;
                 break;
            default:
                 pszDataBits = DATA_BITS_UNKNOWN_STR;
                 break;
         }

         switch( lnctlLineControl.bParity )
         {
            case PARITY_NONE:
                 pszParity = PARITY_NONE_STR;
                 break;
            case PARITY_ODD:
                 pszParity = PARITY_ODD_STR;
                 break;
            case PARITY_EVEN:
                 pszParity = PARITY_EVEN_STR;
                 break;
            case PARITY_MARK:
                 pszParity = PARITY_MARK_STR;
                 break;
            case PARITY_SPACE:
                 pszParity = PARITY_SPACE_STR;
                 break;
            default:
                 pszParity = PARITY_UNKNOWN_STR;
                 break;
         }

         switch( lnctlLineControl.bStopBits )
         {
            case STOP_BITS_1:
                 pszStopBits = STOP_BITS_1_STR;
                 break;
            case STOP_BITS_15:
                 pszStopBits = STOP_BITS_15_STR;
                 break;
            case STOP_BITS_2:
                 pszStopBits = STOP_BITS_2_STR;
                 break;
            default:
                 pszStopBits = STOP_BITS_UNKNOWN_STR;
                 break;
         }

         /* fetch rexx variables details (rexx mallocs, we free) */

         InitVar( &BaudVar, &DataVar, args[1].strptr );
         InitVar( &DataVar, &PariVar, args[2].strptr );
         InitVar( &PariVar, &StopVar, args[3].strptr );
         InitVar( &StopVar, NULL,     args[4].strptr );

         rxrc = RexxVariablePool( &BaudVar ); /* GET the rexx vars   */

         if ( rxrc == RXSHV_MEMFL ) {
            rc = ERROR_NOT_ENOUGH_MEMORY;
         } else if ( rxrc != RXSHV_OK )  {
            rc = ERROR_INVALID_PARAMETER;
         } else {

            /* set rexx variable details with retrieved values */

            UpdtVar( &BaudVar, 3, &(erateBaudRate.ulCurrentBaudRate) );
            UpdtVar( &DataVar, 0, pszDataBits );
            UpdtVar( &PariVar, 0, pszParity );
            UpdtVar( &StopVar, 0, pszStopBits );

            rxrc = RexxVariablePool( &BaudVar ); /* SET the rexx vars*/

            if ( rxrc != RXSHV_OK ) {  /* set port handle var failed */
               rc = ERROR_INVALID_PARAMETER;
            }
         }

         FreeVar( &BaudVar );          /* Cleanup                    */
         FreeVar( &DataVar );
         FreeVar( &PariVar );
         FreeVar( &StopVar );
      }
   }

   BUILDRXDOSRET( retstr, rc );        /* Return DosDevIOCtl result  */

   return( ROUTINE_VALID );
}

/*********************************************************************/
/*                                                                   */
/* Function:  RxAsyncSetLnCtrl                                       */
/*                                                                   */
/* Purpose:   Set status of specified port                           */
/*                                                                   */
/*            Utilises IOCTLs 42H and 43H.                           */
/*                                                                   */
/* Syntax:    call RxAsyncSetLnCtrl port 19200 8 'N' 1               */
/*                                                                   */
/* Params:    port   - handle of open port                           */
/*            baud   - baudrate to set                               */
/*            data   - data bits to set                              */
/*            parity - parity to set                                 */
/*            stop   - stop bits to set                              */
/*                                                                   */
/* Return:                                                           */
/*   Call successful - NO_ERROR (0)                                  */
/*   Call failed     - rc from DosDevIOCtl                           */
/*                                                                   */
/* Example:                                                          */
/*                                                                   */
/*   NewBaud    = 19200                                              */
/*   NewData    = 8                                                  */
/*   NewParity  = 'N'                                                */
/*   NewStop    = 1                                                  */
/*   rc = RxAsyncSetLnCtrl( PortHandle, NewBaud,                     */
/*                          NewData, NewParity, NewStop )            */
/*   if rc <> 0                                                      */
/*     then                                                          */
/*       say 'RxAsyncSetLnCtrl failed with rc=>'rc'<'                */
/*     else                                                          */
/*       say 'RxAsyncSetLnCtrl has set the new settings'             */
/*                                                                   */
/*   say 'Baudrate now >'NewBaud'<'                                  */
/*   say 'Databits now >'NewData'<'                                  */
/*   say 'Parity   now >'NewParity'<'                                */
/*   say 'Stopbits now >'NewStop'<'                                  */
/*                                                                   */
/*********************************************************************/

APIRET RxAsyncSetLnCtrl(
          CHAR     *name,
          ULONG    numargs,
          RXSTRING args[],
          CHAR     *queuename,
          RXSTRING *retstr
       )
{
   APIRET       rc;                    /* Function return code       */
   HFILE        hcomPort;              /* Communications port handle */
   EXTDRATE     erateBaudRate;         /* Extended bitrate params    */
   LINECONTROL  lnctlLineControl;      /* Line control parameters    */

   if ( numargs != 5 )                 /* Must have five arguments   */
      return( ROUTINE_INVALID );

   if ( !RXVALIDSTRING(args[0]) )      /* Must specify port handle   */
      return( ROUTINE_INVALID );

   if ( !Str2Num(args[0].strptr, &hcomPort, STR2NUM_DEC2ULONG) )
      return( ROUTINE_INVALID );       /* Convert handle failed      */

   rc = CallDevice( hcomPort, IOCTL_ASYNC, ASYNC_GETEXTDRATE, NULL, &erateBaudRate );
   if ( rc == NO_ERROR ) {

      rc = CallDevice( hcomPort, IOCTL_ASYNC, ASYNC_GETLINECTRL, NULL, &lnctlLineControl );
      if ( rc == NO_ERROR ) {

         if ( RXVALIDSTRING(args[1]) ) /* If specify baud rate       */
         {
            if ( !Str2Num(args[1].strptr, &(erateBaudRate.ulCurrentBaudRate), STR2NUM_DEC2ULONG) )
               return( ROUTINE_INVALID );       /* Convert baudrate failed    */

            erateBaudRate.bCurrentFraction  = (BYTE)0;
         }

         if ( RXVALIDSTRING(args[2]) ) /* If specify data bits       */
         {
            if        ( strcmp(args[2].strptr,  DATA_BITS_5_STR) == 0 ) {
               lnctlLineControl.bDataBits  = DATA_BITS_5;
            } else if ( strcmp(args[2].strptr,  DATA_BITS_6_STR) == 0 ) {
               lnctlLineControl.bDataBits  = DATA_BITS_6;
            } else if ( strcmp(args[2].strptr,  DATA_BITS_7_STR) == 0 ) {
               lnctlLineControl.bDataBits  = DATA_BITS_7;
            } else if ( strcmp(args[2].strptr,  DATA_BITS_8_STR) == 0 ) {
               lnctlLineControl.bDataBits  = DATA_BITS_8;
            } else {
               BUILDRXDOSRET( retstr, ERROR_INVALID_PARAMETER );
               return( ROUTINE_VALID );
            }
         }

         if ( RXVALIDSTRING(args[3]) ) /* If specify parity          */
         {
            if        ( stricmp(args[3].strptr, PARITY_NONE_STR ) == 0 ) {
               lnctlLineControl.bParity    = PARITY_NONE;
            } else if ( stricmp(args[3].strptr, PARITY_ODD_STR  ) == 0 ) {
               lnctlLineControl.bParity    = PARITY_ODD;
            } else if ( stricmp(args[3].strptr, PARITY_EVEN_STR ) == 0 ) {
               lnctlLineControl.bParity    = PARITY_EVEN;
            } else if ( stricmp(args[3].strptr, PARITY_MARK_STR ) == 0 ) {
               lnctlLineControl.bParity    = PARITY_MARK;
            } else if ( stricmp(args[3].strptr, PARITY_SPACE_STR) == 0 ) {
               lnctlLineControl.bParity    = PARITY_SPACE;
            } else {
               BUILDRXDOSRET( retstr, ERROR_INVALID_PARAMETER );
               return( ROUTINE_VALID );
            }
         }

         if ( RXVALIDSTRING(args[4]) ) /* If specify stop bits       */
         {
            if        ( strcmp(args[4].strptr,  STOP_BITS_1_STR ) == 0 ) {
               lnctlLineControl.bStopBits  = STOP_BITS_1;
            } else if ( strcmp(args[4].strptr,  STOP_BITS_15_STR) == 0 ) {
               lnctlLineControl.bStopBits  = STOP_BITS_15;
            } else if ( strcmp(args[4].strptr,  STOP_BITS_2_STR ) == 0 ) {
               lnctlLineControl.bStopBits  = STOP_BITS_2;
            } else {
               BUILDRXDOSRET( retstr, ERROR_INVALID_PARAMETER );
               return( ROUTINE_VALID );
            }
         }

         rc = CallDevice( hcomPort, IOCTL_ASYNC, ASYNC_SETEXTDRATE, &erateBaudRate, NULL );
         if ( rc == NO_ERROR ) {

            rc = CallDevice( hcomPort, IOCTL_ASYNC, ASYNC_SETLINECTRL, &lnctlLineControl, NULL );
         }
      }
   }

   BUILDRXDOSRET( retstr, rc );        /* Return DosDevIOCtl result  */

   return( ROUTINE_VALID );
}

/*********************************************************************/
/*                                                                   */
/* Function:  RxAsyncSetBreakOff                                     */
/*                                                                   */
/* Purpose:   Set break off                                          */
/*                                                                   */
/*            Utilises IOCTL 45H.                                    */
/*                                                                   */
/* Syntax:    call RxAsyncSetBreakOff handlevar                      */
/*                                                                   */
/* Params:    handlevar - rexx variable with com handle              */
/*                                                                   */
/* Return:                                                           */
/*   Call successful - NO_ERROR (0)                                  */
/*   Call failed     - rc from DosDevIOCtl                           */
/*                                                                   */
/* Example:                                                          */
/*                                                                   */
/*   rc = RxAsyncSetBreakOff( PortHandle )                           */
/*   if rc <> 0                                                      */
/*      then say 'RxAsyncSetBreakOff failed with rc=>'rc'<'          */
/*      else say 'RxAsyncSetBreakOff returned okay'                  */
/*                                                                   */
/*********************************************************************/

APIRET RxAsyncSetBreakOff(
          CHAR     *name,
          ULONG    numargs,
          RXSTRING args[],
          CHAR     *queuename,
          RXSTRING *retstr
       )
{
   APIRET   rc;                        /* Function return code       */
   COMERR   ComError;                  /* Communication Error Status */
   HFILE    hcomPort;                  /* Communications port handle */

   if ( numargs != 1 )                 /* Must have one argument     */
      return( ROUTINE_INVALID );

   if ( !RXVALIDSTRING(args[0]) )      /* Must specify port handle   */
      return( ROUTINE_INVALID );

   if ( !Str2Num(args[0].strptr, &hcomPort, STR2NUM_DEC2ULONG) )
      return( ROUTINE_INVALID );       /* Convert handle failed      */

   rc = CallDevice( hcomPort, IOCTL_ASYNC, ASYNC_SETBREAKOFF, NULL, &ComError );

   BUILDRXDOSRET( retstr, rc );        /* Return DosDevIOCtl result  */

   return( ROUTINE_VALID );
}

/*********************************************************************/
/*                                                                   */
/* Function:  RxAsyncSetBreakOn                                      */
/*                                                                   */
/* Purpose:   Set break on                                           */
/*                                                                   */
/*            Utilises IOCTL 4BH.                                    */
/*                                                                   */
/* Syntax:    call RxAsyncSetBreakOn handlevar                       */
/*                                                                   */
/* Params:    handlevar - rexx variable with com handle              */
/*                                                                   */
/* Return:                                                           */
/*   Call successful - NO_ERROR (0)                                  */
/*   Call failed     - rc from DosDevIOCtl                           */
/*                                                                   */
/* Example:                                                          */
/*                                                                   */
/*   rc = RxAsyncSetBreakOn( PortHandle )                            */
/*   if rc <> 0                                                      */
/*      then say 'RxAsyncSetBreakOn failed with rc=>'rc'<'           */
/*      else say 'RxAsyncSetBreakOn returned okay'                   */
/*                                                                   */
/*********************************************************************/

APIRET RxAsyncSetBreakOn(
          CHAR     *name,
          ULONG    numargs,
          RXSTRING args[],
          CHAR     *queuename,
          RXSTRING *retstr
       )
{
   APIRET   rc;                        /* Function return code       */
   COMERR   ComError;                  /* Communication Error Status */
   HFILE    hcomPort;                  /* Communications port handle */

   if ( numargs != 1 )                 /* Must have one argument     */
      return( ROUTINE_INVALID );

   if ( !RXVALIDSTRING(args[0]) )      /* Must specify port handle   */
      return( ROUTINE_INVALID );

   if ( !Str2Num(args[0].strptr, &hcomPort, STR2NUM_DEC2ULONG) )
      return( ROUTINE_INVALID );       /* Convert handle failed      */

   rc = CallDevice( hcomPort, IOCTL_ASYNC, ASYNC_SETBREAKON, NULL, &ComError );

   BUILDRXDOSRET( retstr, rc );        /* Return DosDevIOCtl result  */

   return( ROUTINE_VALID );
}

/*********************************************************************/
/*                                                                   */
/* Function:  RxAsyncTxImmediate                                     */
/*                                                                   */
/* Purpose:   Transmit specified byte immediately                    */
/*                                                                   */
/*            Utilises IOCTL 44H.                                    */
/*                                                                   */
/* Syntax:    call RxAsyncTxImmediate handlevar byte                 */
/*                                                                   */
/* Params:    handlevar - rexx variable with com handle              */
/*            byte      - byte to be transmitted                     */
/*                                                                   */
/* Return:                                                           */
/*   Call successful - NO_ERROR (0)                                  */
/*   Call failed     - rc from DosDevIOCtl                           */
/*                                                                   */
/* Example:                                                          */
/*                                                                   */
/*   rc = RxAsyncTxImmediate( PortHandle, '00' )                     */
/*   if rc <> 0                                                      */
/*      then say 'RxAsyncTxImmediate failed with rc=>'rc'<'          */
/*      else say 'RxAsyncTxImmediate returned okay'                  */
/*                                                                   */
/*********************************************************************/

APIRET RxAsyncTxImmediate(
          CHAR     *name,
          ULONG    numargs,
          RXSTRING args[],
          CHAR     *queuename,
          RXSTRING *retstr
       )
{
   APIRET   rc;                        /* Function return code       */
   HFILE    hcomPort;                  /* Communications port handle */
   BYTE     bAnyByte;                  /* Byte to be sent            */

   if ( numargs != 2 )                 /* Must have two arguments    */
      return( ROUTINE_INVALID );

   if (   !RXVALIDSTRING(args[0])      /* Must specify port handle   */
       || !RXVALIDSTRING(args[1]) )    /* and byte to send           */
      return( ROUTINE_INVALID );

   if ( !Str2Num(args[0].strptr, &hcomPort, STR2NUM_DEC2ULONG) )
      return( ROUTINE_INVALID );       /* Convert handle failed      */

   if ( !Str2Num(args[1].strptr, &bAnyByte, STR2NUM_HEX2BYTE) )
      return( ROUTINE_INVALID );

   rc = CallDevice( hcomPort, IOCTL_ASYNC, ASYNC_TRANSMITIMM, &bAnyByte, NULL );

   BUILDRXDOSRET( retstr, rc );        /* Return DosDevIOCtl result  */

   return( ROUTINE_VALID );
}

/*********************************************************************/
/*                                                                   */
/* Function:  RxAsyncTxStop                                          */
/*                                                                   */
/* Purpose:   Stop transmission (XOFF)                               */
/*                                                                   */
/*            Utilises IOCTL 47H.                                    */
/*                                                                   */
/* Syntax:    call RxAsyncTxStop handlevar                           */
/*                                                                   */
/* Params:    handlevar  - rexx variable with com handle             */
/*                                                                   */
/* Return:                                                           */
/*   Call successful - NO_ERROR (0)                                  */
/*   Call failed     - rc from DosDevIOCtl                           */
/*                                                                   */
/* Example:                                                          */
/*                                                                   */
/*   rc = RxAsyncTxStop( PortHandle )                                */
/*   if rc <> 0                                                      */
/*      then say 'RxAsyncTxStop failed with rc=>'rc'<'               */
/*      else say 'RxAsyncTxStop returned okay.'                      */
/*                                                                   */
/*********************************************************************/

APIRET RxAsyncTxStop(
          CHAR     *name,
          ULONG    numargs,
          RXSTRING args[],
          CHAR     *queuename,
          RXSTRING *retstr
       )
{
   APIRET   rc;                        /* Function return code       */
   HFILE    hcomPort;                  /* Communications port handle */

   if ( numargs != 1 )                 /* Must have one argument     */
      return( ROUTINE_INVALID );

   if ( !RXVALIDSTRING(args[0]) )      /* Must specify port          */
      return( ROUTINE_INVALID );

   if ( !Str2Num(args[0].strptr, &hcomPort, STR2NUM_DEC2ULONG) )
      return( ROUTINE_INVALID );       /* Convert handle failed      */

   rc = CallDevice( hcomPort, IOCTL_ASYNC, ASYNC_STOPTRANSMIT, NULL, NULL );

   BUILDRXDOSRET( retstr, rc );        /* Return DosDevIOCtl result  */

   return( ROUTINE_VALID );
}

/*********************************************************************/
/*                                                                   */
/* Function:  RxAsyncTxStart                                         */
/*                                                                   */
/* Purpose:   Start transmission (XON)                               */
/*                                                                   */
/*            Utilises IOCTL 48H.                                    */
/*                                                                   */
/* Syntax:    call RxAsyncTxStart handlevar                          */
/*                                                                   */
/* Params:    handlevar  - rexx variable with com handle             */
/*                                                                   */
/* Return:                                                           */
/*   Call successful - NO_ERROR (0)                                  */
/*   Call failed     - rc from DosDevIOCtl                           */
/*                                                                   */
/* Example:                                                          */
/*                                                                   */
/*   rc = RxAsyncTxStart( PortHandle )                               */
/*   if rc <> 0                                                      */
/*      then say 'RxAsyncTxStart failed with rc=>'rc'<'              */
/*      else say 'RxAsyncTxStart returned okay.'                     */
/*                                                                   */
/*********************************************************************/

APIRET RxAsyncTxStart(
          CHAR     *name,
          ULONG    numargs,
          RXSTRING args[],
          CHAR     *queuename,
          RXSTRING *retstr
       )
{
   APIRET   rc;                        /* Function return code       */
   HFILE    hcomPort;                  /* Communications port handle */

   if ( numargs != 1 )                 /* Must have one argument     */
      return( ROUTINE_INVALID );

   if ( !RXVALIDSTRING(args[0]) )      /* Must specify port          */
      return( ROUTINE_INVALID );

   if ( !Str2Num(args[0].strptr, &hcomPort, STR2NUM_DEC2ULONG) )
      return( ROUTINE_INVALID );       /* Convert handle failed      */

   rc = CallDevice( hcomPort, IOCTL_ASYNC, ASYNC_STARTTRANSMIT, NULL, NULL );

   BUILDRXDOSRET( retstr, rc );        /* Return DosDevIOCtl result  */

   return( ROUTINE_VALID );
}

/*********************************************************************/
/*                                                                   */
/* Function:  RxAsyncGetComStatus                                    */
/*                                                                   */
/* Purpose:   Return current COM status                              */
/*                                                                   */
/*            Utilises IOCTL 64H.                                    */
/*                                                                   */
/* Syntax:    call RxAsyncGetComStatus handlevar, statusvar          */
/*                                                                   */
/* Params:    handlevar  - rexx variable with com handle             */
/*            statusvar  - rexx variable to store status value       */
/*                                                                   */
/* Return:                                                           */
/*   Call successful - NO_ERROR (0)                                  */
/*   Call failed     - rc from DosDevIOCtl                           */
/*                                                                   */
/* Example:                                                          */
/*                                                                   */
/*   rc = RxAsyncGetComStatus( PortHandle, 'ComStatus' )             */
/*   if rc <> 0                                                      */
/*      then say 'RxAsyncGetComStatus failed with rc=>'rc'<'         */
/*      else say 'RxAsyncGetComStatus returned okay.'                */
/*                                                                   */
/*********************************************************************/

APIRET RxAsyncGetComStatus(
          CHAR     *name,
          ULONG    numargs,
          RXSTRING args[],
          CHAR     *queuename,
          RXSTRING *retstr
       )
{
   APIRET   rc;                        /* Function return code       */
   HFILE    hcomPort;                  /* Communications port handle */
   BYTE     bComStatus;                /* COM status returned        */
   SHVBLOCK StatusVar;                 /* Rexx var for status        */

   if ( numargs != 2 )                 /* Must have two arguments    */
      return( ROUTINE_INVALID );

   if (   !RXVALIDSTRING(args[0])      /* Must specify port          */
       || !RXVALIDSTRING(args[1]) )    /* and COM status var name    */
      return( ROUTINE_INVALID );

   if ( !Str2Num(args[0].strptr, &hcomPort, STR2NUM_DEC2ULONG) )
      return( ROUTINE_INVALID );       /* Convert handle failed      */

   rc = CallDevice( hcomPort, IOCTL_ASYNC, ASYNC_GETCOMMSTATUS, NULL, &bComStatus );
   if ( rc == NO_ERROR ) {

      rc = SetRxVar( &StatusVar, args[1].strptr, 5, &(bComStatus) );
   }

   BUILDRXDOSRET( retstr, rc );        /* Return DosDevIOCtl result  */

   return( ROUTINE_VALID );
}

/*********************************************************************/
/*                                                                   */
/* Function:  RxAsyncGetTxStatus                                     */
/*                                                                   */
/* Purpose:   Return current Transmit status                         */
/*                                                                   */
/*            Utilises IOCTL 65H.                                    */
/*                                                                   */
/* Syntax:    call RxAsyncGetTxStatus  handlevar, statusvar          */
/*                                                                   */
/* Params:    handlevar  - rexx variable with com handle             */
/*            statusvar  - rexx variable to store status value       */
/*                                                                   */
/* Return:                                                           */
/*   Call successful - NO_ERROR (0)                                  */
/*   Call failed     - rc from DosDevIOCtl                           */
/*                                                                   */
/* Example:                                                          */
/*                                                                   */
/*   rc = RxAsyncGetTxStatus( PortHandle, 'TxStatus' )               */
/*   if rc <> 0                                                      */
/*      then say 'RxAsyncGetTxStatus failed with rc=>'rc'<'          */
/*      else say 'RxAsyncGetTxStatus returned okay.'                 */
/*                                                                   */
/*********************************************************************/

APIRET RxAsyncGetTxStatus(
          CHAR     *name,
          ULONG    numargs,
          RXSTRING args[],
          CHAR     *queuename,
          RXSTRING *retstr
       )
{
   APIRET   rc;                        /* Function return code       */
   HFILE    hcomPort;                  /* Communications port handle */
   BYTE     bTxStatus;                 /* Tx status returned         */
   SHVBLOCK StatusVar;                 /* Rexx var for status        */

   if ( numargs != 2 )                 /* Must have two arguments    */
      return( ROUTINE_INVALID );

   if (   !RXVALIDSTRING(args[0])      /* Must specify port          */
       || !RXVALIDSTRING(args[1]) )    /* and tx status var name     */
      return( ROUTINE_INVALID );

   if ( !Str2Num(args[0].strptr, &hcomPort, STR2NUM_DEC2ULONG) )
      return( ROUTINE_INVALID );       /* Convert handle failed      */

   rc = CallDevice( hcomPort, IOCTL_ASYNC, ASYNC_GETLINESTATUS, NULL, &bTxStatus );
   if ( rc == NO_ERROR ) {

      rc = SetRxVar( &StatusVar, args[1].strptr, 5, &(bTxStatus) );
   }

   BUILDRXDOSRET( retstr, rc );        /* Return DosDevIOCtl result  */

   return( ROUTINE_VALID );
}

/*********************************************************************/
/*                                                                   */
/* Function:  RxAsyncGetMdmOutput                                    */
/*                                                                   */
/* Purpose:   Return current Modem Output signals                    */
/*                                                                   */
/*            Utilises IOCTL 66H.                                    */
/*                                                                   */
/* Syntax:    call RxAsyncGetMdmOutput handlevar, signalvar          */
/*                                                                   */
/* Params:    handlevar  - rexx variable with com handle             */
/*            signalvar  - rexx variable to store signals value      */
/*                                                                   */
/* Return:                                                           */
/*   Call successful - NO_ERROR (0)                                  */
/*   Call failed     - rc from DosDevIOCtl                           */
/*                                                                   */
/* Example:                                                          */
/*                                                                   */
/*   rc = RxAsyncGetMdmOutput( PortHandle, 'MdmOutput' )             */
/*   if rc <> 0                                                      */
/*      then say 'RxAsyncGetMdmOutput failed with rc=>'rc'<'         */
/*      else say 'RxAsyncGetMdmOutput returned okay.'                */
/*                                                                   */
/*********************************************************************/

APIRET RxAsyncGetMdmOutput(
          CHAR     *name,
          ULONG    numargs,
          RXSTRING args[],
          CHAR     *queuename,
          RXSTRING *retstr
       )
{
   APIRET   rc;                        /* Function return code       */
   HFILE    hcomPort;                  /* Communications port handle */
   BYTE     bMdmOutput;                /* Modem Output returned      */
   SHVBLOCK OutputVar;                 /* Rexx var for modem output  */

   if ( numargs != 2 )                 /* Must have two arguments    */
      return( ROUTINE_INVALID );

   if (   !RXVALIDSTRING(args[0])      /* Must specify port          */
       || !RXVALIDSTRING(args[1]) )    /* and modem output var name  */
      return( ROUTINE_INVALID );

   if ( !Str2Num(args[0].strptr, &hcomPort, STR2NUM_DEC2ULONG) )
      return( ROUTINE_INVALID );       /* Convert handle failed      */

   rc = CallDevice( hcomPort, IOCTL_ASYNC, ASYNC_GETMODEMOUTPUT, NULL, &bMdmOutput );
   if ( rc == NO_ERROR ) {

      rc = SetRxVar( &OutputVar, args[1].strptr, 5, &(bMdmOutput) );
   }

   BUILDRXDOSRET( retstr, rc );        /* Return DosDevIOCtl result  */

   return( ROUTINE_VALID );
}

/*********************************************************************/
/*                                                                   */
/* Function:  RxAsyncSetMdmOutput                                    */
/*                                                                   */
/* Purpose:   Alters current Modem Output signals                    */
/*                                                                   */
/*            Utilises IOCTL 46H.                                    */
/*                                                                   */
/* Syntax:    call RxAsyncSetMdmOutput handlevar, onmask, offmask    */
/*                                                                   */
/* Params:    handlevar  - rexx variable with com handle             */
/*            onmask     - on mask setting                           */
/*            offmask    - off mask setting                          */
/*                                                                   */
/* Return:                                                           */
/*   Call successful - NO_ERROR (0)                                  */
/*   Call failed     - rc from DosDevIOCtl                           */
/*                                                                   */
/* Example:                                                          */
/*                                                                   */
/*   none_on  = '00'                                                 */
/*   dtr_on   = '01'                                                 */
/*   rts_on   = '02'                                                 */
/*   both_on  = '03'                                                 */
/*   none_off = 'FF'                                                 */
/*   dtr_off  = 'FE'                                                 */
/*   rts_off  = 'FD'                                                 */
/*   both_off = 'FC'                                                 */
/*   rc = RxAsyncSetMdmOutput( PortHandle, none_on, dtr_off )        */
/*   if rc <> 0                                                      */
/*      then say 'RxAsyncSetMdmOutput failed with rc=>'rc'<'         */
/*      else say 'RxAsyncSetMdmOutput returned okay.'                */
/*                                                                   */
/*********************************************************************/

APIRET RxAsyncSetMdmOutput(
          CHAR     *name,
          ULONG    numargs,
          RXSTRING args[],
          CHAR     *queuename,
          RXSTRING *retstr
       )
{
   APIRET      rc;                     /* Function return code       */
   HFILE       hcomPort;               /* Communications port handle */
   MODEMSTATUS mdmstControl;           /* Modem control              */
   COMERR      ComError;               /* Communication Error Status */

   if ( numargs != 3 )                 /* Must have three arguments  */
      return( ROUTINE_INVALID );

   if (   !RXVALIDSTRING(args[0])      /* Must specify port          */
       || !RXVALIDSTRING(args[1])      /* and signal on mask         */
       || !RXVALIDSTRING(args[2]) )    /* and signal off mask        */
      return( ROUTINE_INVALID );

   if ( !Str2Num(args[0].strptr, &hcomPort, STR2NUM_DEC2ULONG) )
      return( ROUTINE_INVALID );       /* Convert handle failed      */

   if ( !Str2Num(args[1].strptr, &(mdmstControl.fbModemOn), STR2NUM_HEX2BYTE) )
      return( ROUTINE_INVALID );

   if ( !Str2Num(args[2].strptr, &(mdmstControl.fbModemOff), STR2NUM_HEX2BYTE) )
      return( ROUTINE_INVALID );

   rc = CallDevice( hcomPort, IOCTL_ASYNC, ASYNC_SETMODEMCTRL, &mdmstControl, &ComError );

   BUILDRXDOSRET( retstr, rc );        /* Return DosDevIOCtl result  */

   return( ROUTINE_VALID );
}

/*********************************************************************/
/*                                                                   */
/* Function:  RxAsyncGetMdmInput                                     */
/*                                                                   */
/* Purpose:   Return current Modem Input signals                     */
/*                                                                   */
/*            Utilises IOCTL 67H.                                    */
/*                                                                   */
/* Syntax:    call RxAsyncGetMdmInput handlevar, signalvar           */
/*                                                                   */
/* Params:    handlevar  - rexx variable with com handle             */
/*            signalvar  - rexx variable to store signals value      */
/*                                                                   */
/* Return:                                                           */
/*   Call successful - NO_ERROR (0)                                  */
/*   Call failed     - rc from DosDevIOCtl                           */
/*                                                                   */
/* Example:                                                          */
/*                                                                   */
/*   rc = RxAsyncGetMdmInput( PortHandle, 'MdmInput' )               */
/*   if rc <> 0                                                      */
/*      then say 'RxAsyncGetMdmInput failed with rc=>'rc'<'          */
/*      else say 'RxAsyncGetMdmInput returned okay.'                 */
/*                                                                   */
/*********************************************************************/

APIRET RxAsyncGetMdmInput(
          CHAR     *name,
          ULONG    numargs,
          RXSTRING args[],
          CHAR     *queuename,
          RXSTRING *retstr
       )
{
   APIRET   rc;                        /* Function return code       */
   HFILE    hcomPort;                  /* Communications port handle */
   BYTE     bMdmInput;                 /* Modem Input returned       */
   SHVBLOCK InputVar;                  /* Rexx var for modem input   */

   if ( numargs != 2 )                 /* Must have two arguments    */
      return( ROUTINE_INVALID );

   if (   !RXVALIDSTRING(args[0])      /* Must specify port          */
       || !RXVALIDSTRING(args[1]) )    /* and modem input var name   */
      return( ROUTINE_INVALID );

   if ( !Str2Num(args[0].strptr, &hcomPort, STR2NUM_DEC2ULONG) )
      return( ROUTINE_INVALID );       /* Convert handle failed      */

   rc = CallDevice( hcomPort, IOCTL_ASYNC, ASYNC_GETMODEMINPUT, NULL, &bMdmInput );
   if ( rc == NO_ERROR ) {

      rc = SetRxVar( &InputVar, args[1].strptr, 5, &(bMdmInput) );
   }

   BUILDRXDOSRET( retstr, rc );        /* Return DosDevIOCtl result  */

   return( ROUTINE_VALID );
}

/*********************************************************************/
/*                                                                   */
/* Function:  RxAsyncGetRxCount                                      */
/*                                                                   */
/* Purpose:   Return current RX queue size and count                 */
/*                                                                   */
/*            Utilises IOCTL 68H.                                    */
/*                                                                   */
/* Syntax:    call RxAsyncGetRxCount  handlevar, countvar, sizevar   */
/*                                                                   */
/* Params:    handlevar  - rexx variable with com handle             */
/*            countvar   - rexx variable to store count value        */
/*            sizevar    - rexx variable to store size  value        */
/*                                                                   */
/* Return:                                                           */
/*   Call successful - NO_ERROR (0)                                  */
/*   Call failed     - rc from DosDevIOCtl                           */
/*                                                                   */
/* Example:                                                          */
/*                                                                   */
/*   rc = RxAsyncGetRxCount( PortHandle, 'RxCount', 'RxSize' )       */
/*   if rc <> 0                                                      */
/*      then say 'RxAsyncGetRxCount failed with rc=>'rc'<'           */
/*      else say 'RxAsyncGetRxCount returned okay.'                  */
/*                                                                   */
/*********************************************************************/

APIRET RxAsyncGetRxCount(
          CHAR     *name,
          ULONG    numargs,
          RXSTRING args[],
          CHAR     *queuename,
          RXSTRING *retstr
       )
{
   APIRET   rc;                        /* Function return code       */
   HFILE    hcomPort;                  /* Communications port handle */
   IOQUEUE  ioqueStatus;               /* IO Queue details returned  */
   SHVBLOCK CountVar;                  /* Rexx var for queue count   */
   SHVBLOCK SizeVar;                   /* Rexx var for queue size    */

   if ( numargs != 3 )                 /* Must have three arguments  */
      return( ROUTINE_INVALID );

   if (   !RXVALIDSTRING(args[0])      /* Must specify port          */
       || !RXVALIDSTRING(args[1])      /* and queue count var name   */
       || !RXVALIDSTRING(args[2]) )    /* and queue size var name    */
      return( ROUTINE_INVALID );

   if ( !Str2Num(args[0].strptr, &hcomPort, STR2NUM_DEC2ULONG) )
      return( ROUTINE_INVALID );       /* Convert handle failed      */

   rc = CallDevice( hcomPort, IOCTL_ASYNC, ASYNC_GETINQUECOUNT, NULL, &ioqueStatus );
   if ( rc == NO_ERROR ) {

      rc = SetRxVar( &CountVar, args[1].strptr, 2, &(ioqueStatus.usQueueCount) );
      if ( rc == NO_ERROR ) {

         rc = SetRxVar( &SizeVar,  args[2].strptr, 2, &(ioqueStatus.usQueueSize) );
      }
   }

   BUILDRXDOSRET( retstr, rc );        /* Return DosDevIOCtl result  */

   return( ROUTINE_VALID );
}

/*********************************************************************/
/*                                                                   */
/* Function:  RxAsyncGetTxCount                                      */
/*                                                                   */
/* Purpose:   Return current TX queue size and count                 */
/*                                                                   */
/*            Utilises IOCTL 69H.                                    */
/*                                                                   */
/* Syntax:    call RxAsyncGetTxCount  handlevar, countvar, sizevar   */
/*                                                                   */
/* Params:    handlevar  - rexx variable with com handle             */
/*            countvar   - rexx variable to store count value        */
/*            sizevar    - rexx variable to store size  value        */
/*                                                                   */
/* Return:                                                           */
/*   Call successful - NO_ERROR (0)                                  */
/*   Call failed     - rc from DosDevIOCtl                           */
/*                                                                   */
/* Example:                                                          */
/*                                                                   */
/*   rc = RxAsyncGetTxCount( PortHandle, 'TxCount', 'TxSize' )       */
/*   if rc <> 0                                                      */
/*      then say 'RxAsyncGetTxCount failed with rc=>'rc'<'           */
/*      else say 'RxAsyncGetTxCount returned okay.'                  */
/*                                                                   */
/*********************************************************************/

APIRET RxAsyncGetTxCount(
          CHAR     *name,
          ULONG    numargs,
          RXSTRING args[],
          CHAR     *queuename,
          RXSTRING *retstr
       )
{
   APIRET   rc;                        /* Function return code       */
   HFILE    hcomPort;                  /* Communications port handle */
   IOQUEUE  ioqueStatus;               /* IO Queue details returned  */
   SHVBLOCK CountVar;                  /* Rexx var for queue count   */
   SHVBLOCK SizeVar;                   /* Rexx var for queue size    */

   if ( numargs != 3 )                 /* Must have three arguments  */
      return( ROUTINE_INVALID );

   if (   !RXVALIDSTRING(args[0])      /* Must specify port          */
       || !RXVALIDSTRING(args[1])      /* and queue count var name   */
       || !RXVALIDSTRING(args[2]) )    /* and queue size var name    */
      return( ROUTINE_INVALID );

   if ( !Str2Num(args[0].strptr, &hcomPort, STR2NUM_DEC2ULONG) )
      return( ROUTINE_INVALID );       /* Convert handle failed      */

   rc = CallDevice( hcomPort, IOCTL_ASYNC, ASYNC_GETOUTQUECOUNT, NULL, &ioqueStatus );
   if ( rc == NO_ERROR ) {

      rc = SetRxVar( &CountVar, args[1].strptr, 2, &(ioqueStatus.usQueueCount) );
      if ( rc == NO_ERROR ) {

         rc = SetRxVar( &SizeVar,  args[2].strptr, 2, &(ioqueStatus.usQueueSize) );
      }
   }

   BUILDRXDOSRET( retstr, rc );        /* Return DosDevIOCtl result  */

   return( ROUTINE_VALID );
}

/*********************************************************************/
/*                                                                   */
/* Function:  RxAsyncGetComError                                     */
/*                                                                   */
/* Purpose:   Return current COM Error setting                       */
/*                                                                   */
/*            Utilises IOCTL 6DH.                                    */
/*                                                                   */
/* Syntax:    call RxAsyncGetComError handlevar, errorvar            */
/*                                                                   */
/* Params:    handlevar  - rexx variable with com handle             */
/*            errorvar   - rexx variable to store error value        */
/*                                                                   */
/* Return:                                                           */
/*   Call successful - NO_ERROR (0)                                  */
/*   Call failed     - rc from DosDevIOCtl                           */
/*                                                                   */
/* Example:                                                          */
/*                                                                   */
/*   rc = RxAsyncGetComError( PortHandle, 'ComError' )               */
/*   if rc <> 0                                                      */
/*      then say 'RxAsyncGetComError failed with rc=>'rc'<'          */
/*      else say 'RxAsyncGetComError returned okay.'                 */
/*                                                                   */
/*********************************************************************/

APIRET RxAsyncGetComError(
          CHAR     *name,
          ULONG    numargs,
          RXSTRING args[],
          CHAR     *queuename,
          RXSTRING *retstr
       )
{
   APIRET   rc;                        /* Function return code       */
   HFILE    hcomPort;                  /* Communications port handle */
   USHORT   usComError;                /* COM Error returned         */
   SHVBLOCK ErrorVar;                  /* Rexx var for com error     */

   if ( numargs != 2 )                 /* Must have two arguments    */
      return( ROUTINE_INVALID );

   if (   !RXVALIDSTRING(args[0])      /* Must specify port          */
       || !RXVALIDSTRING(args[1]) )    /* and com error var name     */
      return( ROUTINE_INVALID );

   if ( !Str2Num(args[0].strptr, &hcomPort, STR2NUM_DEC2ULONG) )
      return( ROUTINE_INVALID );       /* Convert handle failed      */

   rc = CallDevice( hcomPort, IOCTL_ASYNC, ASYNC_GETCOMMERROR, NULL, &usComError );
   if ( rc == NO_ERROR ) {

      rc = SetRxVar( &ErrorVar, args[1].strptr, 6, &(usComError) );
   }

   BUILDRXDOSRET( retstr, rc );        /* Return DosDevIOCtl result  */

   return( ROUTINE_VALID );
}

/*********************************************************************/
/*                                                                   */
/* Function:  RxAsyncGetComEvent                                     */
/*                                                                   */
/* Purpose:   Return current COM Event setting                       */
/*                                                                   */
/*            Utilises IOCTL 72H.                                    */
/*                                                                   */
/* Syntax:    call RxAsyncGetComEvent handlevar, eventvar            */
/*                                                                   */
/* Params:    handlevar  - rexx variable with com handle             */
/*            eventvar   - rexx variable to store event value        */
/*                                                                   */
/* Return:                                                           */
/*   Call successful - NO_ERROR (0)                                  */
/*   Call failed     - rc from DosDevIOCtl                           */
/*                                                                   */
/* Example:                                                          */
/*                                                                   */
/*   rc = RxAsyncGetComEvent( PortHandle, 'ComEvent' )               */
/*   if rc <> 0                                                      */
/*      then say 'RxAsyncGetComEvent failed with rc=>'rc'<'          */
/*      else say 'RxAsyncGetComEvent returned okay.'                 */
/*                                                                   */
/*********************************************************************/

APIRET RxAsyncGetComEvent(
          CHAR     *name,
          ULONG    numargs,
          RXSTRING args[],
          CHAR     *queuename,
          RXSTRING *retstr
       )
{
   APIRET   rc;                        /* Function return code       */
   HFILE    hcomPort;                  /* Communications port handle */
   USHORT   usComEvent;                /* COM Event returned         */
   SHVBLOCK EventVar;                  /* Rexx var for com event     */

   if ( numargs != 2 )                 /* Must have two arguments    */
      return( ROUTINE_INVALID );

   if (   !RXVALIDSTRING(args[0])      /* Must specify port          */
       || !RXVALIDSTRING(args[1]) )    /* and com event var name     */
      return( ROUTINE_INVALID );

   if ( !Str2Num(args[0].strptr, &hcomPort, STR2NUM_DEC2ULONG) )
      return( ROUTINE_INVALID );       /* Convert handle failed      */

   rc = CallDevice( hcomPort, IOCTL_ASYNC, ASYNC_GETCOMMEVENT, NULL, &usComEvent );
   if ( rc == NO_ERROR ) {

      rc = SetRxVar( &EventVar, args[1].strptr, 6, &(usComEvent) );
   }

   BUILDRXDOSRET( retstr, rc );        /* Return DosDevIOCtl result  */

   return( ROUTINE_VALID );
}

/*********************************************************************/
/*                                                                   */
/* Function:  RxAsyncOpen                                            */
/*                                                                   */
/* Purpose:   Open the specified port                                */
/*                                                                   */
/* Syntax:    call RxAsyncOpen name 'handlevar'                      */
/*                                                                   */
/* Params:    name      - name of com port or named pipe             */
/*            handlevar - rexx variable to store handle in           */
/*                                                                   */
/* Return:                                                           */
/*   Call successful - NO_ERROR (0)                                  */
/*   Call failed     - rc from DosOpen                               */
/*                                                                   */
/* Example:                                                          */
/*                                                                   */
/*   PortName   = 'COM4'                                             */
/*   PortHandle = ''                                                 */
/*   rc = RxAsyncOpen( PortName, 'PortHandle' )                      */
/*   if rc <> 0                                                      */
/*      then say 'RxAsyncOpen failed with rc=>'rc'<'                 */
/*      else say 'RxAsyncOpen opened device handle=>'PortHandle'<'   */
/*                                                                   */
/*********************************************************************/

APIRET RxAsyncOpen(
          CHAR     *name,
          ULONG    numargs,
          RXSTRING args[],
          CHAR     *queuename,
          RXSTRING *retstr
       )
{
   APIRET   rc;                        /* Function return code       */
   HFILE    hcomPort;                  /* Communications port handle */
   ULONG    ulOpenAction;              /* Open action                */
   SHVBLOCK PortVar;                   /* Rexx variable for handle   */

   if ( numargs != 2 )                 /* Must have two arguments    */
      return( ROUTINE_INVALID );

   if (   !RXVALIDSTRING(args[0])      /* Must specify port name     */
       || !RXVALIDSTRING(args[1]) )    /* and handle var name        */
      return( ROUTINE_INVALID );

   hcomPort = 0L;

   rc = DosOpen( args[0].strptr,
                 &hcomPort,
                 &ulOpenAction,
                 0,
                 FILE_NORMAL,
                 FILE_OPEN,
                 OPEN_ACCESS_READWRITE | OPEN_SHARE_DENYNONE,
                 (PEAOP2) NULL );

   if ( rc != NO_ERROR ) {             /* Couldn't open port         */
      fprintf( stdout, "\nDosOpen() failed with RC=%d.", rc );
   } else {

      rc = SetRxVar( &PortVar, args[1].strptr, 4, &hcomPort );
      if ( rc != NO_ERROR ) {          /* Couldn't continue          */
         DosClose( hcomPort );
      }
   }

   BUILDRXDOSRET( retstr, rc );        /* Return DosOpen result      */

   return( ROUTINE_VALID );
}

/*********************************************************************/
/*                                                                   */
/* Function:  RxAsyncClose                                           */
/*                                                                   */
/* Purpose:   Close the specified port                               */
/*                                                                   */
/* Syntax:    call RxAsyncClose handlevar                            */
/*                                                                   */
/* Params:    handlevar - rexx variable with com handle              */
/*                                                                   */
/* Return:                                                           */
/*   Call successful - NO_ERROR (0)                                  */
/*   Call failed     - rc from DosClose                              */
/*                                                                   */
/* Example:                                                          */
/*                                                                   */
/*   rc = RxAsyncClose( PortHandle )                                 */
/*   if rc <> 0                                                      */
/*      then say 'RxAsyncClose failed with rc=>'rc'<'                */
/*      else say 'RxAsyncClose closed the device'                    */
/*                                                                   */
/*********************************************************************/

APIRET RxAsyncClose(
          CHAR     *name,
          ULONG    numargs,
          RXSTRING args[],
          CHAR     *queuename,
          RXSTRING *retstr
       )
{
   APIRET   rc;                        /* Function return code       */
   HFILE    hcomPort;                  /* Communications port handle */

   if ( numargs != 1 )                 /* Must have one argument     */
      return( ROUTINE_INVALID );

   if ( !RXVALIDSTRING(args[0]) )      /* Must specify port          */
      return( ROUTINE_INVALID );

   if ( !Str2Num(args[0].strptr, &hcomPort, STR2NUM_DEC2ULONG) )
      return( ROUTINE_INVALID );       /* Convert handle failed      */

   rc = DosClose( hcomPort );
   if ( rc != NO_ERROR ) {
      fprintf( stdout, "\nDosClose() failed with RC=%d.", rc );
   }

   BUILDRXDOSRET( retstr, rc );        /* Return DosClose result     */

   return( ROUTINE_VALID );
}

/*********************************************************************/
/*                                                                   */
/* Function:  RxAsyncWait                                            */
/*                                                                   */
/* Purpose:   Wait for input from the specified port                 */
/*                                                                   */
/*            Utilises IOCTL 72H.                                    */
/*                                                                   */
/* Syntax:    call RxAsyncWait port timeout                          */
/*                                                                   */
/* Params:    port    - handle of open port                          */
/*            timeout - maximum period to wait in milliseconds       */
/*                      -1 = no timeout                              */
/*                                                                   */
/* Return:                                                           */
/*   Data detected    - NO_ERROR (0)                                 */
/*   Wait timed out   - RXASYNC_TIMED_OUT (-1)                       */
/*   Error occurred   - rc from DosDevIOCtl(ASYNC_GETCOMMEVENT)      */
/*                                                                   */
/* Example:                                                          */
/*                                                                   */
/*   say 'Wait for data 'time()                                      */
/*   rc = RxAsyncWait( PortHandle, 3000 )                            */
/*   say 'Waited enough 'time()                                      */
/*   if rc = 0                                                       */
/*      then say 'RxAsyncWait detected input'                        */
/*      else if rc = -1                                              */
/*              then say 'RxAsyncWait timed out'                     */
/*              else say 'RxAsyncWait returned rc=>'rc'<'            */
/*                                                                   */
/*********************************************************************/

APIRET RxAsyncWait(
          CHAR     *name,
          ULONG    numargs,
          RXSTRING args[],
          CHAR     *queuename,
          RXSTRING *retstr
       )
{
   APIRET       rc;                    /* Function return code       */
   HFILE        hcomPort;              /* Communications port handle */
   LONG         lTimeOut;              /* Time in hundredths         */
   USHORT       usComEvent;            /* COM Event Information      */
   BOOL         fQueueEmpty;           /* Input queue is empty       */
   BOOL         fTimedOut;             /* Function has timed out     */
   LONG         lElapsed;              /* Time in milliseconds       */
   DATETIME     tTimeStart;            /* Timer start time           */
   DATETIME     tTimeNow;              /* Time now                   */

   if ( numargs != 2 )                 /* Must have two arguments    */
      return( ROUTINE_INVALID );

   if (   !RXVALIDSTRING(args[0])      /* Must specify port handle   */
       || !RXVALIDSTRING(args[1]) )    /* and timeout                */
      return( ROUTINE_INVALID );

   if ( !Str2Num(args[0].strptr, &hcomPort, STR2NUM_DEC2ULONG) )
      return( ROUTINE_INVALID );       /* Convert handle failed      */

   if ( !Str2Num(args[1].strptr, &lTimeOut, STR2NUM_DEC2LONG) )
      return( ROUTINE_INVALID );       /* Convert timeout failed     */

   fQueueEmpty  = TRUE;
   fTimedOut    = FALSE;

   DosGetDateTime( &tTimeStart );

   do {

      rc = CallDevice( hcomPort, IOCTL_ASYNC, ASYNC_GETCOMMEVENT, NULL, &usComEvent );

      BUILDRXDOSRET( retstr, rc );     /* Return DosDevIOCtl result  */

      if ( rc != NO_ERROR ) {          /* Call failed                */

         return( ROUTINE_VALID );

      } else if ( usComEvent & CHAR_RECEIVED ) {

         fQueueEmpty = FALSE;

      } else if ( lTimeOut == 0 ) {    /* just test and return       */

         fTimedOut = TRUE;

      } else if ( lTimeOut < 0 ) {     /* no timeout required        */

         DosSleep( MAX_SLEEP );        /* wait 1/100th second        */

      } else {

         DosGetDateTime( &tTimeNow );
         lElapsed = MyDiffTime( &tTimeNow, &tTimeStart );

         if ( (lElapsed < 0) || (lElapsed > lTimeOut) ) {
            fTimedOut = TRUE;
         } else {
            DosSleep( MAX_SLEEP );     /* wait 1/100th second        */
         }
      }

   } while( fQueueEmpty && !fTimedOut );

   if ( fQueueEmpty ) {
      BUILDRXDOSRET( retstr, RXASYNC_TIMED_OUT );
   }

   return( ROUTINE_VALID );            /* All ok                     */
}

/*********************************************************************/
/*                                                                   */
/* Function:  RxAsyncRead                                            */
/*                                                                   */
/* Purpose:   Read from the specified port                           */
/*                                                                   */
/*            Utilises IOCTL 72H.                                    */
/*                                                                   */
/* Syntax:    rc = RxAsyncRead( port, length, timeout, 'varname' )   */
/*                                                                   */
/* Params:    port    - handle of open port                          */
/*            length  - number of bytes to read, 0=a line, n=chars   */
/*            timeout - maximum period to wait in milliseconds       */
/*                      -1 = no timeout                              */
/*            varname - name of rexx variable to return data in      */
/*                                                                   */
/* Return:                                                           */
/*   Call successful - NO_ERROR (0)                                  */
/*   Call failed     - rc from DosRead                               */
/*                                                                   */
/* Example:                                                          */
/*                                                                   */
/*   InpStr = ''                                                     */
/*   rc =  RxAsyncRead( PortHandle, 0, 3000, 'InpStr' )              */
/*   if rc <> 0                                                      */
/*      then say 'RxAsyncRead failed with rc=>'rc'<'                 */
/*      else say 'RxAsyncRead returned >'InpStr'<'                   */
/*                                                                   */
/*********************************************************************/

APIRET RxAsyncRead(
          CHAR     *name,
          ULONG    numargs,
          RXSTRING args[],
          CHAR     *queuename,
          RXSTRING *retstr
       )
{
   APIRET       rc;                    /* Function return code       */
   APIRET       rxrc;                  /* RexxVariablePool return    */
   HFILE        hcomPort;              /* Communications port handle */
   USHORT       usComEvent;            /* COM Event Information      */
   BYTE         ReadBuff[RXAUTOBUFLEN];/* Input buffer               */
   SHVBLOCK     ReadVar;               /* Rexx variable for input    */
   BOOL         fWaitForMore;          /* Poll for more input        */
   BOOL         fTimedOut;             /* Function has timed out     */
   BOOL         fStopReading;          /* Got all we wanted flag     */
   BOOL         fByteFound;            /* Found some data to read    */
   LONG         lNumRequested;         /* Read length (0=line mode)  */
   ULONG        ulNumToRead;           /* Number to read this time   */
   ULONG        ulActuallyRead;        /* Number of chars we did get */
   ULONG        ulTotalRead;           /* Total number of chars read */
   LONG         lElapsed;              /* Time elapsed in millisecs  */
   LONG         lTimeLimit;            /* Timeout in milliseconds    */
   DATETIME     tTimeStart;            /* Timer start time           */
   DATETIME     tTimeNow;              /* Time now                   */

   if ( numargs != 4 )                 /* Must have four arguments   */
      return( ROUTINE_INVALID );

   if (   !RXVALIDSTRING(args[0])      /* Must specify port handle   */
       || !RXVALIDSTRING(args[1])      /* and read length            */
       || !RXVALIDSTRING(args[2])      /* and timeout                */
       || !RXVALIDSTRING(args[3]) )    /* and string to return in    */
      return( ROUTINE_INVALID );

   if ( !Str2Num(args[0].strptr, &hcomPort, STR2NUM_DEC2ULONG) )
      return( ROUTINE_INVALID );       /* Convert handle failed      */

   if ( !Str2Num(args[1].strptr, &lNumRequested, STR2NUM_DEC2LONG) )
      return( ROUTINE_INVALID );       /* Convert length failed      */

   if ( lNumRequested > RXAUTOBUFLEN-1 )
      return( ROUTINE_INVALID );       /* Invalid length specified   */

   if ( !Str2Num(args[2].strptr, &lTimeLimit, STR2NUM_DEC2LONG) )
      return( ROUTINE_INVALID );       /* Convert timeout failed     */

   BUILDRXDOSRET( retstr, NO_ERROR );  /* Initialise return value    */

   /* fetch rexx variable details  (rexx mallocs, we free) */

   InitVar( &ReadVar, NULL, args[3].strptr );

   rxrc = RexxVariablePool( &ReadVar ); /* GET the rexx variable vals*/

   if ( rxrc == RXSHV_MEMFL ) {        /* get vars failed no memory  */
      BUILDRXDOSRET( retstr, ERROR_NOT_ENOUGH_MEMORY );
      FreeVar( &ReadVar );
      return( ROUTINE_VALID );
   }

   if ( rxrc != RXSHV_OK ) {           /* get vars failed            */
      BUILDRXDOSRET( retstr, ERROR_INVALID_PARAMETER );
      FreeVar( &ReadVar );
      return( ROUTINE_VALID );
   }

   strcpy( ReadVar.shvvalue.strptr, "" ); /* Initialise return var   */

   ulTotalRead  = 0;
   fTimedOut    = FALSE;
   fStopReading = FALSE;

   if ( lNumRequested > 0 ) {          /* character mode (blocked)   */
      ulNumToRead = lNumRequested;
   } else  {                           /* line mode (char by char)   */
      ulNumToRead = 1;
   }

   DosGetDateTime( &tTimeStart );

   do {

      fWaitForMore = FALSE;

      do {

         ulActuallyRead = 0;

         memset( &ReadBuff, (BYTE)0, RXAUTOBUFLEN );   /* zeroise */

         rc = DosRead( hcomPort,
                       &ReadBuff,
                       (ULONG) ulNumToRead,
                       &ulActuallyRead );

         if (rc != NO_ERROR) {
            if (rc != ERROR_MORE_DATA) {

               fprintf( stdout, "\nDosRead() failed with RC=%d.", rc );
               FreeVar( &ReadVar );

               BUILDRXDOSRET( retstr, rc );
               return( ROUTINE_VALID );
            }
         }

         if ( ulActuallyRead != ulNumToRead ) {

            fWaitForMore = TRUE;
         }

         if ( ulActuallyRead > 0 ) {

            /*?##### remove nulls from buffer here #####?*/

            ulActuallyRead = strlen( (PSZ) &ReadBuff );

            if ( ulActuallyRead > 0 ) {

               ulTotalRead = ulTotalRead + ulActuallyRead;
               strncat( ReadVar.shvvalue.strptr, (PSZ) &ReadBuff, ulActuallyRead );

               if ( lNumRequested > 0 ) {       /* Character mode */

                  ulNumToRead = lNumRequested - ulTotalRead;
                  if ( ulNumToRead <= 0 ) {
                     fStopReading = TRUE;
                  }

               } else {                              /* Line mode */

                  if ( ReadBuff[0] == (BYTE) 10 ) {      /* LINEFEED */
                     fStopReading = TRUE;
                  } else if ( ulTotalRead >= RXAUTOBUFLEN-1 ) {
                     fStopReading = TRUE;
                  }
               }

            } else if ( (lTimeLimit < 0) || fWaitForMore ) {
               /*? either don't care about timeout ?*/
               /*? or will handle in polling loop! ?*/

            } else {

               DosGetDateTime( &tTimeNow );
               lElapsed = MyDiffTime( &tTimeNow, &tTimeStart );
               if ( lElapsed > lTimeLimit ) {
                  fTimedOut = TRUE;
               }

            } /* data was NOT all nulls */

         } /* data WAS read */

      } while( !fStopReading && !fTimedOut && !fWaitForMore );

      if ( fWaitForMore ) {

         fByteFound = FALSE;

         do {

            rc = CallDevice( hcomPort, IOCTL_ASYNC, ASYNC_GETCOMMEVENT, NULL, &usComEvent );

            if ( rc != NO_ERROR ) {       /* Call failed                */

               FreeVar( &ReadVar );

               BUILDRXDOSRET( retstr, rc );
               return( ROUTINE_VALID );

            } else if ( usComEvent & CHAR_RECEIVED ) {

               fByteFound = TRUE;

            } else if ( lTimeLimit < 0 ) {
               /*? we did say no timeout didn't we! ?*/

               DosSleep( MAX_SLEEP );     /* Wait 1/100th second        */

            } else {

               DosGetDateTime( &tTimeNow );
               lElapsed = MyDiffTime( &tTimeNow, &tTimeStart );

               if ( lElapsed > lTimeLimit ) {
                  fTimedOut = TRUE;
               } else {
                  DosSleep( MAX_SLEEP );  /* Wait 1/100th second        */
               }
            }

         } while( !fByteFound && !fTimedOut );

      } /* wait for more */

   } while( !fStopReading && !fTimedOut );

   /* set rexx variable details with retrieved values */

   ReadVar.shvcode            = RXSHV_SYSET;
   ReadVar.shvret             = (UCHAR) 0;
   ReadVar.shvvalue.strlength = strlen( ReadVar.shvvalue.strptr );
   ReadVar.shvvaluelen        = strlen( ReadVar.shvvalue.strptr );

   rxrc = RexxVariablePool( &ReadVar ); /* SET the rexx variable vals*/

   FreeVar( &ReadVar );                /* Cleanup                    */

   if ( rxrc != RXSHV_OK ) {           /* set variable value failed  */
      BUILDRXDOSRET( retstr, ERROR_INVALID_PARAMETER );
   } else if ( !fStopReading ) {
      BUILDRXDOSRET( retstr, RXASYNC_TIMED_OUT );
   }

   return( ROUTINE_VALID );            /* All ok                     */
}

/*********************************************************************/
/*                                                                   */
/* Function:  RxAsyncWrite                                           */
/*                                                                   */
/* Purpose:   Write to the specified port                            */
/*                                                                   */
/* Syntax:    rc = RxAsyncWrite( port, 0, "ATZ"d2c(13), 'Remain' )   */
/*                                                                   */
/* Params:    port   - handle of open port                           */
/*            delay  - delay before writing                          */
/*            outvar - string to be written                          */
/*            remvar - name of variable to return unwritten count in */
/*                                                                   */
/* Return:                                                           */
/*   Call successful - NO_ERROR (0)                                  */
/*   Call failed     - rc from DosWrite                              */
/*                                                                   */
/* Example:                                                          */
/*                                                                   */
/*   crlf = D2C(13)''D2C(10)                                         */
/*   Remaining = 0                                                   */
/*   rc = RxAsyncWrite( PortHandle, 0, 'ATZ'crlf, 'Remaining' )      */
/*   if rc <> 0                                                      */
/*      then say 'Modem not turned on, >'Remaining'< chars unwritten'*/
/*      else say 'RxAsyncWrite output ATZ<crlf>'                     */
/*                                                                   */
/*********************************************************************/

APIRET RxAsyncWrite(
          CHAR     *name,
          ULONG    numargs,
          RXSTRING args[],
          CHAR     *queuename,
          RXSTRING *retstr
       )
{
   APIRET       rc1;                   /* Function return code       */
   APIRET       rc2;                   /* RexxVariablePool return    */
   HFILE        hcomPort;              /* Communications port handle */
   ULONG        ulWritten;             /* Number of chars written    */
   ULONG        ulRemains;             /* Number of chars not written*/
   SHVBLOCK     LeftVar;               /* Rexx variable for num left */
   LONG         millisecs;             /* Time to sleep              */

   if ( numargs != 4 )                 /* Must have four arguments   */
      return( ROUTINE_INVALID );

   if (   !RXVALIDSTRING(args[0])      /* Must specify port handle   */
       || !RXVALIDSTRING(args[1])      /* and write delay            */
       ||   RXNULLSTRING(args[2])      /* and var to write           */
       || !RXVALIDSTRING(args[3]) )    /* and var to return num left */
      return( ROUTINE_INVALID );

   if ( !Str2Num(args[0].strptr, &hcomPort, STR2NUM_DEC2ULONG) )
      return( ROUTINE_INVALID );       /* Convert handle failed      */

   if ( !Str2Num(args[1].strptr, &millisecs, STR2NUM_DEC2LONG) )
      return( ROUTINE_INVALID );       /* Convert millisecs failed   */

   if ( millisecs < 0 )                /* Can not wait negative time */
      return( ROUTINE_INVALID );

   rc1 = NO_ERROR;                     /* Initialise return value    */

   if ( millisecs > 0 )                /* Only if its worthwile      */
      DosSleep( millisecs );

   ulWritten = 0;

   if ( !RXZEROLENSTRING(args[2]) ) {  /* Not empty string           */

      rc1 = DosWrite( hcomPort,
                      args[2].strptr,
                      args[2].strlength,
                      &ulWritten );

      if ( rc1 != NO_ERROR ) {
         fprintf( stdout, "\nDosWrite() failed with RC=%d.", rc1 );
      }
   }

   ulRemains = args[2].strlength - ulWritten;

   rc2 = SetRxVar( &LeftVar, args[3].strptr, 3, &ulRemains );
   if ( rc2 != NO_ERROR ) {            /* Couldn't set rexx variable */
      if ( rc1 == NO_ERROR ) {         /* Not already in error       */
         rc1 = rc2;
      }
   }

   BUILDRXDOSRET( retstr, rc1 );       /* Return DosWrite result     */

   return( ROUTINE_VALID );
}

/*********************************************************************/
/*                                                                   */
/* Function:  RxAsyncPrint                                           */
/*                                                                   */
/* Purpose:   Write to stdout                                        */
/*                                                                   */
/* Syntax:    call RxAsyncPrint string                               */
/*                                                                   */
/* Params:    string  - text to be written                           */
/*                                                                   */
/* Return:                                                           */
/*   Call successful - NO_ERROR (0)                                  */
/*                                                                   */
/* Example:                                                          */
/*                                                                   */
/*   call RxAsyncPrint 'This is printed on stdout'                   */
/*                                                                   */
/*********************************************************************/

APIRET RxAsyncPrint(
          CHAR     *name,
          ULONG    numargs,
          RXSTRING args[],
          CHAR     *queuename,
          RXSTRING *retstr )
{
   APIRET       rc;                    /* Function return code       */

   if ( numargs != 1 )                 /* Must have one argument     */
      return( ROUTINE_INVALID );

   if (     RXNULLSTRING(args[0]) )    /* Must be a string (empty ok)*/
      return( ROUTINE_INVALID );

   fprintf( stdout, args[0].strptr );

   BUILDRXDOSRET( retstr, NO_ERROR );  /* Initialise return value    */

   return( ROUTINE_VALID );
}

/*********************************************************************/
/*                                                                   */
/* Function:  RxAsyncCarrier                                         */
/*                                                                   */
/* Purpose:   Test for data carrier detect on specified port         */
/*                                                                   */
/*            Utilises IOCTL 67H.                                    */
/*                                                                   */
/* Syntax:    rc = RxAsyncCarrier( port )                            */
/*                                                                   */
/* Params:    port    - handle of open port                          */
/*                                                                   */
/* Return:                                                           */
/*   Carrier active   - NO_ERROR (0)                                 */
/*   Carrier inactive - RXASYNC_NO_SIGNAL (-2)                       */
/*   Error occurred   - rc from DosDevIOCtl(ASYNC_GETMODEMINPUT)     */
/*                                                                   */
/* Example:                                                          */
/*                                                                   */
/*   rc = RxAsyncCarrier( PortHandle )                               */
/*   if rc = 0                                                       */
/*      then say 'RxAsyncCarrier detected DCD'                       */
/*      else if rc = -2                                              */
/*              then say 'RxAsyncCarrier did not detect DCD'         */
/*              else say 'RxAsyncCarrier failed with rc=>'rc'<'      */
/*                                                                   */
/*********************************************************************/

APIRET RxAsyncCarrier(
          CHAR     *name,
          ULONG    numargs,
          RXSTRING args[],
          CHAR     *queuename,
          RXSTRING *retstr )
{
   HFILE        hcomPort;              /* Communications port handle */

   if ( numargs != 1 )                 /* Must have one argument     */
      return( ROUTINE_INVALID );

   if (   !RXVALIDSTRING(args[0]) )    /* Must specify port handle   */
      return( ROUTINE_INVALID );

   if ( !Str2Num(args[0].strptr, &hcomPort, STR2NUM_DEC2ULONG) )
      return( ROUTINE_INVALID );       /* Convert handle failed      */

   BUILDRXDOSRET( retstr, ModemInput(&hcomPort,DCD_ON) );

   return( ROUTINE_VALID );
}

/*********************************************************************/
/*                                                                   */
/* Function:  RxAsyncBreak                                           */
/*                                                                   */
/* Purpose:   Send break for specified length of time                */
/*                                                                   */
/*            Utilises IOCTLs 45H and 4BH.                           */
/*                                                                   */
/* Syntax:    call RxAsyncBreak handlevar milliseconds               */
/*                                                                   */
/* Params:    handlevar - rexx variable with com handle              */
/*            millisecs - duration of break to send                  */
/*                                                                   */
/* Return:                                                           */
/*   Call successful - NO_ERROR (0)                                  */
/*   Call failed     - rc from DosDevIOCtl                           */
/*                                                                   */
/* Example:                                                          */
/*                                                                   */
/*   BreakTime = 500                                                 */
/*   rc = RxAsyncBreak( PortHandle, BreakTime )                      */
/*   if rc <> 0                                                      */
/*      then say 'RxAsyncBreak failed with rc=>'rc'<'                */
/*      else say 'RxAsyncBreak sent break for 'BreakTime' millisecs' */
/*                                                                   */
/*********************************************************************/

APIRET RxAsyncBreak(
          CHAR     *name,
          ULONG    numargs,
          RXSTRING args[],
          CHAR     *queuename,
          RXSTRING *retstr
       )
{
   APIRET   rc1;                       /* Function return code       */
   APIRET   rc2;                       /* Function return code       */
   COMERR   ComError;                  /* Communication Error Status */
   HFILE    hcomPort;                  /* Communications port handle */
   LONG     millisecs;                 /* Break duration             */

   if ( numargs != 2 )                 /* Must have two arguments    */
      return( ROUTINE_INVALID );

   if (   !RXVALIDSTRING(args[0])      /* Must specify port handle   */
       || !RXVALIDSTRING(args[1]) )    /* and break duration         */
      return( ROUTINE_INVALID );

   if ( !Str2Num(args[0].strptr, &hcomPort, STR2NUM_DEC2ULONG) )
      return( ROUTINE_INVALID );       /* Convert handle failed      */

   if ( !Str2Num(args[1].strptr, &millisecs, STR2NUM_DEC2LONG) )
      return( ROUTINE_INVALID );       /* Convert millisecs failed   */

   if ( millisecs < 0 )                /* Can not break negative time*/
      return( ROUTINE_INVALID );

   rc1 = NO_ERROR;                     /* Initialise return value    */

   if ( millisecs > 0 ) {              /* Only if its worthwile      */

      rc1 = CallDevice( hcomPort, IOCTL_ASYNC, ASYNC_SETBREAKON, NULL, &ComError );
      if ( rc1 == NO_ERROR ) {
         DosSleep( millisecs );
      }

      rc2 = CallDevice( hcomPort, IOCTL_ASYNC, ASYNC_SETBREAKOFF, NULL, &ComError );
      if ( rc2 != NO_ERROR ) {
         if ( rc1 == NO_ERROR ) {      /* Not already in error       */
            rc1 = rc2;
         }
      }
   }

   BUILDRXDOSRET( retstr, rc1 );       /* Return DosDevIOCtl result  */

   return( ROUTINE_VALID );
}

/*********************************************************************/
/*                                                                   */
/* Function:  RxAsyncDcbInfo                                         */
/*                                                                   */
/* Purpose:   Print current port settings                            */
/*                                                                   */
/*            Utilises IOCTL 73H.                                    */
/*                                                                   */
/* Syntax:    call RxAsyncDcbInfo handlevar                          */
/*                                                                   */
/* Params:    handlevar - rexx variable with com handle              */
/*                                                                   */
/* Return:                                                           */
/*   Call successful - NO_ERROR (0)                                  */
/*   Call failed     - rc from DosDevIOCtl                           */
/*                                                                   */
/* Example:                                                          */
/*                                                                   */
/*   rc = RxAsyncDcbInfo( PortHandle )                               */
/*   if rc <> 0                                                      */
/*      then say 'RxAsyncDcbInfo failed with rc=>'rc'<'              */
/*      else say 'RxAsyncDcbInfo returned okay.'                     */
/*                                                                   */
/*********************************************************************/

APIRET RxAsyncDcbInfo(
          CHAR     *name,
          ULONG    numargs,
          RXSTRING args[],
          CHAR     *queuename,
          RXSTRING *retstr
       )
{
   APIRET   rc;                        /* Function return code       */
   HFILE    hcomPort;                  /* Communications port handle */
   DCBINFO2 dcbPortInfo;               /* Device control block       */

   if ( numargs != 1 )                 /* Must have one argument     */
      return( ROUTINE_INVALID );

   if ( !RXVALIDSTRING(args[0]) )      /* Must specify port          */
      return( ROUTINE_INVALID );

   if ( !Str2Num(args[0].strptr, &hcomPort, STR2NUM_DEC2ULONG) )
      return( ROUTINE_INVALID );       /* Convert handle failed      */

   rc = CallDevice( hcomPort, IOCTL_ASYNC, ASYNC_GETDCBINFO, NULL, &dcbPortInfo );

   PrtDcbInfo( &dcbPortInfo );

   BUILDRXDOSRET( retstr, rc );        /* Return DosDevIOCtl result  */

   return( ROUTINE_VALID );
}

/*********************************************************************/
/*                                                                   */
/* Function:  RxAsyncFileExists                                      */
/*                                                                   */
/* Purpose:   Test for existence of specified file                   */
/*                                                                   */
/* Syntax:    if RxAsyncFileExists( FileName )                       */
/*                                                                   */
/* Params:    FileName - Name of file to test for (WILDCARDS OK)     */
/*                                                                   */
/* Return:                                                           */
/*   File was found   - RXASYNC_TRUE (1)                             */
/*   File not found   - RXASYNC_FALSE (0)                            */
/*                                                                   */
/* Example:                                                          */
/*                                                                   */
/*   if RxAsyncFileExists( 'C:\*.BAT' )                              */
/*      then say 'RxAsyncFileExists found a batch file.'             */
/*                                                                   */
/*********************************************************************/

APIRET RxAsyncFileExists(
          CHAR     *name,
          ULONG    numargs,
          RXSTRING args[],
          CHAR     *queuename,
          RXSTRING *retstr )
{
   if ( numargs != 1 )                 /* Must have one argument     */
      return( ROUTINE_INVALID );

   if (   !RXVALIDSTRING(args[0]) )    /* Must specify file name     */
      return( ROUTINE_INVALID );

   if ( FileExists( args[0].strptr ) ) {
      BUILDRXDOSRET( retstr, RXASYNC_TRUE );
   } else {
      BUILDRXDOSRET( retstr, RXASYNC_FALSE );
   }

   return( ROUTINE_VALID );
}

/*********************************************************************/
/*                                                                   */
/* Function:  RxAsyncFileDelete                                      */
/*                                                                   */
/* Purpose:   Delete specified file                                  */
/*                                                                   */
/* Syntax:    if RxAsyncFileDelete( FileName )                       */
/*                                                                   */
/* Params:    FileName - Name of file to delete (NO WILDCARDS)       */
/*                                                                   */
/* Return:                                                           */
/*   File was deleted - RXASYNC_TRUE (1)                             */
/*   File not deleted - RXASYNC_FALSE (0)                            */
/*                                                                   */
/* Example:                                                          */
/*                                                                   */
/*   if RxAsyncFileDelete( 'C:\FRED.TXT' )                           */
/*      then say 'RxAsyncFileDelete deleted file Fred.Txt'           */
/*                                                                   */
/*********************************************************************/

APIRET RxAsyncFileDelete(
          CHAR     *name,
          ULONG    numargs,
          RXSTRING args[],
          CHAR     *queuename,
          RXSTRING *retstr )
{
   if ( numargs != 1 )                 /* Must have one argument     */
      return( ROUTINE_INVALID );

   if (   !RXVALIDSTRING(args[0]) )    /* Must specify file name     */
      return( ROUTINE_INVALID );

   if ( FileDelete( args[0].strptr ) ) {
      BUILDRXDOSRET( retstr, RXASYNC_TRUE );
   } else {
      BUILDRXDOSRET( retstr, RXASYNC_FALSE );
   }

   return( ROUTINE_VALID );
}

/*********************************************************************/
/*                                                                   */
/* Function:  RxAsyncFileCopy                                        */
/*                                                                   */
/* Purpose:   Copy a specified file                                  */
/*                                                                   */
/* Syntax:    if RxAsyncFileCopy( FromName, ToName )                 */
/*                                                                   */
/* Params:    FromName - Name of file to copy (NO WILDCARDS)         */
/*            ToName   - Name to copy to      (NO WILDCARDS)         */
/*                                                                   */
/* Return:                                                           */
/*   File was copied  - RXASYNC_TRUE (1)                             */
/*   File not copied  - RXASYNC_FALSE (0)                            */
/*                                                                   */
/* Example:                                                          */
/*                                                                   */
/*   if RxAsyncFileCopy( 'C:\FRED.TXT', 'D:\FRED.BAK' )              */
/*      then say 'RxAsyncFileCopy copied Fred.Txt to Fred.Bak'       */
/*                                                                   */
/*********************************************************************/

APIRET RxAsyncFileCopy(
          CHAR     *name,
          ULONG    numargs,
          RXSTRING args[],
          CHAR     *queuename,
          RXSTRING *retstr )
{
   if ( numargs != 2 )                 /* Must have two arguments    */
      return( ROUTINE_INVALID );

   if (   !RXVALIDSTRING(args[0])      /* Must specify file name src */
       || !RXVALIDSTRING(args[1]) )    /* Must specify file name trg */
      return( ROUTINE_INVALID );

   if ( FileCopy( args[0].strptr, args[1].strptr ) ) {
      BUILDRXDOSRET( retstr, RXASYNC_TRUE );
   } else {
      BUILDRXDOSRET( retstr, RXASYNC_FALSE );
   }

   return( ROUTINE_VALID );
}

/*********************************************************************/
/*                                                                   */
/* Function:  RxAsyncFileMove                                        */
/*                                                                   */
/* Purpose:   Move a specified file                                  */
/*                                                                   */
/* Syntax:    if RxAsyncFileMove( FromName, ToName )                 */
/*                                                                   */
/* Params:    FromName - Name of file to move (NO WILDCARDS)         */
/*            ToName   - Name to move to      (NO WILDCARDS)         */
/*                                                                   */
/* Return:                                                           */
/*   File was copied  - RXASYNC_TRUE (1)                             */
/*   File not copied  - RXASYNC_FALSE (0)                            */
/*                                                                   */
/* Notes:                                                            */
/*   Permits the movement of files accross drives.                   */
/*                                                                   */
/* Example:                                                          */
/*                                                                   */
/*   if RxAsyncFileMove( 'C:\FRED.TXT', 'D:\FRED.TXT' )              */
/*      then say 'RxAsyncFileMove moved C:\Fred.Txt to D:\Fred.Txt'  */
/*                                                                   */
/*********************************************************************/

APIRET RxAsyncFileMove(
          CHAR     *name,
          ULONG    numargs,
          RXSTRING args[],
          CHAR     *queuename,
          RXSTRING *retstr )
{
   if ( numargs != 2 )                 /* Must have two arguments    */
      return( ROUTINE_INVALID );

   if (   !RXVALIDSTRING(args[0])      /* Must specify file name src */
       || !RXVALIDSTRING(args[1]) )    /* Must specify file name trg */
      return( ROUTINE_INVALID );

   if ( FileMove( args[0].strptr, args[1].strptr ) ) {
      BUILDRXDOSRET( retstr, RXASYNC_TRUE );
   } else {
      BUILDRXDOSRET( retstr, RXASYNC_FALSE );
   }

   return( ROUTINE_VALID );
}

/*********************************************************************/
/*                                                                   */
/* Function:  RxAsyncFileRename                                      */
/*                                                                   */
/* Purpose:   Rename specified file                                  */
/*                                                                   */
/* Syntax:    if RxAsyncFileRename( FromName, ToName )               */
/*                                                                   */
/* Params:    FromName - Name of file to rename  (NO WILDCARDS)      */
/*            ToName   - Name to rename to       (NO WILDCARDS)      */
/*                                                                   */
/* Return:                                                           */
/*   File was renamed - RXASYNC_TRUE (1)                             */
/*   File not renamed - RXASYNC_FALSE (0)                            */
/*                                                                   */
/* Note:                                                             */
/*   Uses return string as holder for target path and file name.     */
/*                                                                   */
/* Example:                                                          */
/*                                                                   */
/*   if RxAsyncFileRename( 'C:\FRED.TXT', 'C:\FRED.OLD' )            */
/*      then say 'RxAsyncFileRename renamed Fred.Txt to Fred.Old'    */
/*                                                                   */
/*********************************************************************/

APIRET RxAsyncFileRename(
          CHAR     *name,
          ULONG    numargs,
          RXSTRING args[],
          CHAR     *queuename,
          RXSTRING *retstr
       )
{
   PSZ          pszPath;
   PSZ          pszName;
   PSZ          pszColon;
   PSZ          pszSlash;

   if ( numargs != 2 )                 /* Must have two arguments    */
      return( ROUTINE_INVALID );

   if (   !RXVALIDSTRING(args[0])      /* Must specify file name old */
       || !RXVALIDSTRING(args[1]) )    /* Must specify file name new */
      return( ROUTINE_INVALID );

   strcpy( retstr->strptr, args[0].strptr );
   pszPath  = retstr->strptr;

   pszColon = strrchr( pszPath, RXASYNC_DRIVE_DELIM_CH );
   pszSlash = strrchr( pszPath, RXASYNC_PATH_DELIM_CH );

   if (pszSlash != NULL) {
      pszColon = pszSlash;
   }
   if (pszColon != NULL) {
      pszColon++;
      pszPath = pszColon;
   }

   pszName  = args[1].strptr;

   pszColon = strrchr( pszName, RXASYNC_DRIVE_DELIM_CH );
   pszSlash = strrchr( pszName, RXASYNC_PATH_DELIM_CH );

   if (pszSlash != NULL) {
      pszColon = pszSlash;
   }

   if (pszColon != NULL) {
      pszColon++;
      pszName = pszColon;
   }

   strcpy( pszPath, pszName );

   if ( FileRename( args[0].strptr, retstr->strptr ) ) {
      BUILDRXDOSRET( retstr, RXASYNC_TRUE );
   } else {
      BUILDRXDOSRET( retstr, RXASYNC_FALSE );
   }

   return( ROUTINE_VALID );            /* All ok                     */
}

/*********************************************************************/
/*                                                                   */
/* Function:  RxAsyncFileNameIs                                      */
/*                                                                   */
/* Purpose:   Determine file name                                    */
/*                                                                   */
/* Syntax:    Tname = RxAsyncFileNameIs( FileName )                  */
/*                                                                   */
/* Params:    FileName - File spec to extract name from              */
/*                                                                   */
/* Return:                                                           */
/*   File name extracted if any                                      */
/*                                                                   */
/* Example:                                                          */
/*                                                                   */
/*   Fname = RxAsyncFileNameIs( 'C:\FRED.TXT' )                      */
/*   say 'RxAsyncFileNameIs returned 'Fname                          */
/*                                                                   */
/*   RxAsyncFileNameIs returned FRED.TXT                             */
/*                                                                   */
/*********************************************************************/

APIRET RxAsyncFileNameIs(
          CHAR     *name,
          ULONG    numargs,
          RXSTRING args[],
          CHAR     *queuename,
          RXSTRING *retstr
       )
{
   PSZ          pszName;
   PSZ          pszColon;
   PSZ          pszSlash;

   if ( numargs != 1 )                 /* Must have one argument     */
      return( ROUTINE_INVALID );

   if (   !RXVALIDSTRING(args[0]) )    /* Must specify file name     */
      return( ROUTINE_INVALID );

   pszName  = args[0].strptr;

   pszColon = strrchr( pszName, RXASYNC_DRIVE_DELIM_CH );
   pszSlash = strrchr( pszName, RXASYNC_PATH_DELIM_CH );

   if (pszSlash != NULL) {
      pszColon = pszSlash;
   }

   if (pszColon != NULL) {
      pszColon++;
      pszName = pszColon;
   }

   BUILDRXSTRING( retstr, pszName );

   return( ROUTINE_VALID );
}

/*********************************************************************/
/*                                                                   */
/* Function:  RxAsyncFilePathIs                                      */
/*                                                                   */
/* Purpose:   Determine path name                                    */
/*                                                                   */
/* Syntax:    Tpath = RxAsyncFilePathIs( FileName )                  */
/*                                                                   */
/* Params:    FileName - File spec to extract path from              */
/*                                                                   */
/* Return:                                                           */
/*   File path extracted if any                                      */
/*                                                                   */
/* Example:                                                          */
/*                                                                   */
/*   Fpath = RxAsyncFilePathIs( 'C:\FRED.TXT' )                      */
/*   say 'RxAsyncFilePathIs returned 'Fpath                          */
/*                                                                   */
/*   RxAsyncFilePathIs returned C:\                                  */
/*                                                                   */
/*********************************************************************/

APIRET RxAsyncFilePathIs(
          CHAR     *name,
          ULONG    numargs,
          RXSTRING args[],
          CHAR     *queuename,
          RXSTRING *retstr
       )
{
   PSZ          pszPath;
   PSZ          pszColon;
   PSZ          pszSlash;

   if ( numargs != 1 )                 /* Must have one argument     */
      return( ROUTINE_INVALID );

   if (   !RXVALIDSTRING(args[0]) )    /* Must specify file name     */
      return( ROUTINE_INVALID );

   strcpy( retstr->strptr, args[0].strptr );
   pszPath  = retstr->strptr;

   pszColon = strrchr( pszPath, RXASYNC_DRIVE_DELIM_CH );
   pszSlash = strrchr( pszPath, RXASYNC_PATH_DELIM_CH );

   if (pszSlash != NULL) {
      pszColon = pszSlash;
   }

   if (pszColon != NULL) {
      pszColon++;
   } else {
      pszColon = pszPath;
   }

   pszColon[0] = RXASYNC_EOSTR_CH;

   retstr->strlength = strlen(pszPath);

   return( ROUTINE_VALID );
}

/*********************************************************************/
/*                                                                   */
/* Function:  RxAsyncDirectory                                       */
/*                                                                   */
/* Purpose:   Query/Change directory                                 */
/*                                                                   */
/* Syntax:    Tdir = RxAsyncDirectory( NewDir )                      */
/*                                                                   */
/* Params:    NewDir - Name of directory to change to if specified   */
/*                                                                   */
/* Return:                                                           */
/*   Current/New directory name                                      */
/*                                                                   */
/* Notes:                                                            */
/*   Differs from the standard directory() function by always        */
/*   appending a trailing backslash to the returned path spec.       */
/*                                                                   */
/* Example:                                                          */
/*                                                                   */
/*   OldDir = RxAsyncDirectory()                                     */
/*   NewDir = 'C:\OS2\'                                              */
/*   CurDir = RxAsyncDirectory( NewDir )                             */
/*                                                                   */
/*********************************************************************/

APIRET RxAsyncDirectory(
          CHAR     *name,
          ULONG    numargs,
          RXSTRING args[],
          CHAR     *queuename,
          RXSTRING *retstr
       )
{
   ULONG   ulDrive;     /* Default drive number (A=1,..,Z=26)           */
   ULONG   ulMap;       /* Drive map area (bits 0-25 indicate validity) */
   UCHAR   ucPath[256]; /* Directory path buffer                        */
   ULONG   ulPathLen;   /* Directory path buffer length                 */
   PSZ     pszPath;     /* Pointer to path buffer                       */
   APIRET  rc;          /* API return code                              */

   pszPath = &ucPath[0];

   if (numargs > 1)                    /* May have one argument/none */
      return( ROUTINE_INVALID );

   if (numargs == 1) {                 /* If argument specified      */

      if ( !RXVALIDSTRING(args[0]) )   /* Must be valid string       */
         return( ROUTINE_INVALID );

      strcpy( pszPath, args[0].strptr );
      /* fprintf( stdout, "\nChanging to >%s<", pszPath ); */

      ulPathLen = strlen( pszPath );

      /* remove trailing backslash if present and not root */
      if ( ulPathLen > 1 ) {
         if (   (pszPath[ulPathLen-1] == RXASYNC_PATH_DELIM_CH)
             && (pszPath[ulPathLen-2] != RXASYNC_DRIVE_DELIM_CH) ) {
            pszPath[ulPathLen-1] = RXASYNC_EOSTR_CH;
         }
      }

      if ( pszPath[1] == RXASYNC_DRIVE_DELIM_CH ) {

         ulDrive = (ULONG)( (BYTE)pszPath[0] - (BYTE)'A' ) + 1;

         /* fprintf( stdout, "\nDosSetDefaultDisk with >%d<", ulDrive ); */
         DosError( FERR_DISABLEHARDERR | FERR_DISABLEEXCEPTION );
         rc = DosSetDefaultDisk( ulDrive );
         DosError( FERR_ENABLEHARDERR | FERR_ENABLEEXCEPTION );
         if ( rc != NO_ERROR ) {
            fprintf( stdout, "\nDosSetDefaultDisk( %d ) failed with RC=%d.", ulDrive, rc );
         }
      }

      /* fprintf( stdout, "\nDosSetCurrentDir with >%s<", pszPath ); */
      rc = DosSetCurrentDir( pszPath );
      if ( rc != NO_ERROR ) {
         fprintf( stdout, "\nDosSetCurrentDir('%s') failed with RC=%d.", pszPath, rc );
      }
   }

   ulPathLen = RXAUTOBUFLEN;

   rc = DosQueryCurrentDisk( &ulDrive, &ulMap );
   /* fprintf( stdout, "\nDosQueryCurrentDisk returned >%d<", ulDrive ); */
   if ( rc != NO_ERROR ) {
      fprintf( stdout, "\nDosQueryCurrentDisk() failed with RC=%d.", rc );
   } else {

      rc = DosQueryCurrentDir( ulDrive, pszPath, &ulPathLen );
      /* fprintf( stdout, "\nDosQueryCurrentDir returned >%s<, >%d<", pszPath, ulPathLen ); */
      if ( rc != NO_ERROR ) {
         fprintf( stdout, "\nDosQueryCurrentDir() failed with RC=%d.", rc );
      } else {

         /* return drive spec */
         retstr->strptr[0] = (CHAR)( (BYTE)'A' + (BYTE)(ulDrive-1) );
         retstr->strptr[1] = RXASYNC_DRIVE_DELIM_CH;
         retstr->strptr[2] = RXASYNC_PATH_DELIM_CH;
         retstr->strptr[3] = RXASYNC_EOSTR_CH;

         /* append path if any */
         if (strlen( pszPath ) > 0) {

            strcat( retstr->strptr, pszPath );
            ulPathLen = strlen( retstr->strptr );

            /* append trailing backslash if room */
            if (ulPathLen < RXAUTOBUFLEN) {
               retstr->strptr[ulPathLen] = RXASYNC_PATH_DELIM_CH;
               ulPathLen++;
               retstr->strptr[ulPathLen] = RXASYNC_EOSTR_CH;
            }
         }
      }
   }

   retstr->strlength = strlen(retstr->strptr);

   return( ROUTINE_VALID );
}

/*********************************************************************/
/* END MODULE RXASYNC.c                                              */
/*********************************************************************/
